Package: groHMM
Version: 1.47.0
Date: 2025-07-24
Title: GRO-seq Analysis Pipeline
Authors@R: c(person("Charles G.", "Danko", role = c("aut"),
                email="dankoc@gmail.com"),
            person("Minho", "Chae", role = c("aut"),
                email="minho.chae@gmail.com"),
            person("Andre", "Martins", role = c("ctb"),
                email="alm253@cornell.edu"),
            person("W. Lee", "Kraus", role = c("aut", "fnd"),
                email="lee.kraus@utsouthwestern.edu"),
	    person("Anusha", "Nagari", role = c("ctb"),
                email="anusha.nagari@utsouthwestern.edu"),
	    person("Tulip", "Nandu", role = c("cre", "ctb"),
                email="tulip.nandu@utsouthwestern.edu"),
	    person("Pariksheet", "Nanda", role = c("ctb"),
                email="pan79@pitt.edu",
		comment = c(ORCID = "0000-0001-9726-4552")))
Depends: R (>= 4.1.0), MASS, parallel, S4Vectors (>= 0.17.25), IRanges
        (>= 2.13.12), Seqinfo, GenomicRanges (>= 1.31.8),
        GenomicAlignments (>= 1.15.6), rtracklayer (>= 1.39.7)
Suggests: BiocStyle, GenomicFeatures, edgeR, org.Hs.eg.db,
        TxDb.Hsapiens.UCSC.hg19.knownGene
Description: A pipeline for the analysis of GRO-seq data.
URL: https://github.com/Kraus-Lab/groHMM
BugReports: https://github.com/Kraus-Lab/groHMM/issues
License: GPL-3
biocViews: Sequencing, Software
LazyLoad: yes
RoxygenNote: 7.3.2
git_url: https://git.bioconductor.org/packages/groHMM
git_branch: devel
git_last_commit: 1ac2709
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-05-18
NeedsCompilation: yes
Packaged: 2026-05-18 22:17:17 UTC; biocbuild
Author: Charles G. Danko [aut],
  Minho Chae [aut],
  Andre Martins [ctb],
  W. Lee Kraus [aut, fnd],
  Anusha Nagari [ctb],
  Tulip Nandu [cre, ctb],
  Pariksheet Nanda [ctb] (ORCID: <https://orcid.org/0000-0001-9726-4552>)
Maintainer: Tulip Nandu <tulip.nandu@utsouthwestern.edu>
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-19 04:54:06 UTC; unix

useDynLib(groHMM)
import(MASS, parallel, S4Vectors, IRanges, Seqinfo, GenomicRanges, GenomicAlignments, rtracklayer)
export(writeWiggle, detectTranscripts, makeConsensusAnnotations,
    evaluateHMMInAnnotations, windowAnalysis, breakTranscriptsOnGenes,
    combineTranscripts, getTxDensity, limitToXkb, runMetaGene, metaGene,
    pausingIndex, polymeraseWave, getCores, averagePlot,
    expressedGenes, metaGeneMatrix, metaGene_nL, readBed)
 importFrom("grDevices", "rgb")
  importFrom("graphics", "abline", "axis", "hist", "par", "plot.new",
             "plot.window", "rect", "text", "title")
  importFrom("stats", "dexp", "dgamma", "dnorm", "fisher.test",
             "ks.test", "loess", "ppois", "predict", "qnorm", "smooth")
  importFrom("utils", "read.table")

###########################################################################
##
##   Copyright 2013, 2014 Charles Danko and Minho Chae.
##
##   This program is part of the groHMM R package
##
##   groHMM is free software: you can redistribute it and/or modify it
##   under the terms of the GNU General Public License as published by
##   the Free Software Foundation, either version 3 of the License, or
##   (at your option) any later version.
##
##   This program is distributed in the hope that it will be useful, but
##   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
##   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
##   for more details.
##
##   You should have received a copy of the GNU General Public License along
##   with this program.  If not, see <http://www.gnu.org/licenses/>.
##
##########################################################################

########################################################################
##
##  Distribution Fitting Functions -- MLEfit given X.
##  Date: 2014-03-27
##
##  Presently, fits:
##      -- Gamma distribution  (RgammaMLE)
##      -- Normal distrubution (Rnorm)
##
##  Todo:
##  (1) Add fits for: 
##      -- Normal by MLE
##      -- Poisson
##      -- Negative binomial (??)
##
########################################################################


#' RgammaMLE fits a gamma distribution to a specified data vector using maximum
#' likelihood.
#'
#' @param X A vector of observations, assumed to be real numbers in the
#' inveraval [0,+Inf).
#' @return Returns a list of parameters for the best-fit gamma
#' distribution (shape and scale).
#' @author Charles G. Danko
RgammaMLE <- function(X) {
    if(sum(X<0) > 0) message("Negative values not allowed!")
    N <- as.double(NROW(X))
    sumxis <- as.double(sum(X))
    sumlogxis <- as.double(sum(log(X)))
    Fit <- .Call("RgammaMLE", N, sumxis, sumlogxis, PACKAGE = "groHMM")
    return(Fit)
}

#' Rnorm fits a normal distribution to a specified data vector using maximum
#' likelihood.
#'
#' @param X A vector of observations, assumed to be real numbers in the
#' inveraval (-Inf,+Inf).
#' @return Returns a list of parameters for the best-fit normal distribution
#' (mean and varience).
#' @author Charles G. Danko
Rnorm <- function(X) {
    returnList      <- list()
    returnList$mean <- mean(X)
    returnList$var  <- var(X)

    return(returnList)
}

################################
##
##  R interface to MLEFit for alpha*Normal+(1-alpha)*Exponential hybrid 
##  distribution.
##
################################

#' Rnorm.exp fits a normal+exponential distribution to a specified data
#' vector using maximum likelihood.
#'
#' Distrubtion function devined by: alpha*Normal(mean, varience)+(1-alpha)
#' *Exponential(lambda).
#'
#' Fits nicely with data types that look normal overall, but have a long
#' tail starting for positive values.
#'
#' @param xi A vector of observations, assumed to be real numbers in the
#' inveraval (-Inf,+Inf).
#' @param wi A vector of weights.  Default: vector of repeating 1; indicating
#' all observations are weighted equally. (Are these normalized internally?!
#' Or do they have to be [0,1]?)
#' @param guess Initial guess for paremeters.  Default: c(0.5, 0, 1, 1).
#' @param tol Convergence tolerance.  Default: sqrt(.Machine$double.eps).
#' @param maxit  Maximum number of iterations.  Default: 10,000.
#' @return Returns a list of parameters for the best-fit normal distribution
#' (alpha, mean, varience, and lambda).
#' @author Charles G. Danko
Rnorm.exp <- function(xi, wi=rep(1,NROW(xi)), guess=c(0.5, 0, 1, 1), 
    tol=sqrt(.Machine$double.eps), maxit=10000) {
    Fit <- .Call("RNormExpMLE", xi, wi, guess, tol, as.integer(maxit), 
        PACKAGE = "groHMM")
}

################################
##
## weighted.var -- Computes the weighted varience, where the varience is 
## weighted by some factor...
##
###############################
weighted.var <- function(x, w, na.rm = FALSE) {
    if (na.rm) {
        w <- w[i <- !is.na(x)]
        x <- x[i]
    }
    sum.w <- sum(w)
    sum.w2 <- sum(w^2)
    mean.w <- sum(x * w) / sum(w)
    (sum.w / (sum.w^2 - sum.w2)) * sum(w * (x - mean.w)^2, na.rm = na.rm)
}

###########################################################################
##
##   Copyright 2009, 2010, 2011, 2012, 2013 Charles Danko and Minho Chae.
##
##   This program is part of the groHMM R package
##
##   groHMM is free software: you can redistribute it and/or modify it 
##   under the terms of the GNU General Public License as published by 
##   the Free Software Foundation, either version 3 of the License, or  
##   (at your option) any later version.
##
##   This program is distributed in the hope that it will be useful, but 
##   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
##   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
##   for more details.
##
##   You should have received a copy of the GNU General Public License along 
##   with this program.  If not, see <http://www.gnu.org/licenses/>.
##
##########################################################################

#' limitToXkb truncates a set of genomic itnervals at a constant, maximum size.
#'
#' @param features A GRanges object representing a set of genomic coordinates.
#' The meta-plot will be centered on the start position.
#' @param offset Starts the interval from this position relative to the start
#' of each genomic features.
#' @param size Specifies the size of the window.
#' @return Returns GRanges object with new genomic coordiates.
#' @author Minho Chae and Charles G. Danko
#' @examples
#' tx <- GRanges("chr7", IRanges(1000, 30000), strand="+")
#' newTX <- limitToXkb(tx)
##  This function limits a genomic range to a samll region relative to the 
## transcription site.
limitToXkb <- function(features, offset=1000, size=13000) {
    w <- width(features)

    # 1. do nothing for w < offset 
    # 2. offset < w and w < size
    small  <- (offset < w) & (w < size)
    if (any(small)) { 
        features[small,] <- flank(features[small,], -1*(w[small]-offset), 
            start=FALSE)
    }

    # 2. w > size
    big  <- w > size 
    if (any(big)) {
        features[big,] <- resize(features[big,], width=size)

        bigPlus <- big & as.character(strand(features))=="+"
        if (any(bigPlus)) 
            start(features[bigPlus,]) <- start(features[bigPlus,]) + offset 

        bigMinus <- big & as.character(strand(features))=="-"
        if (any(bigMinus)) 
            end(features[bigMinus,]) <- end(features[bigMinus,]) - offset 
    }

    return(features)
}

#' countMappableReadsInInterval counts the number of mappable reads in a set
#' of genomic features.
#'
#' Supports parallel processing using mclapply in the 'parallel' package.
#' To change the number of processors, use the argument 'mc.cores'.
#'
#' @param features A GRanges object representing a set of genomic coordinates.
#' The meta-plot will be centered on the start position.
#' @param UnMap List object representing the position of un-mappable reads.
#' Default: not used.
#' @param debug If set to TRUE, provides additional print options.
#' Default: FALSE
#' @param ... Extra argument passed to mclapply
#' @return Returns a vector of counts, each representing the number of reads
#' inside each genomic interval.
#' @author Charles G. Danko and Minho Chae
##
##  This function takes information from BED file to represent regions 
## (as in CountReadsInInterval), and  a list structure representing unmappable 
## positions.  Counts the number of mappable positions in  the interval.
##      f  == features in a BED format, where columns represent: 
##            Chr, Start, End, Strand, ID.
##      UnMAQ  == List structure of the un-mappable bases in the genome.
##
##  Addumptions:
##  (1) FeatureStart and FeatureEnd should be vectors ONLY on the same 
##  chromosome as PROBEStart!!!
##  (2) Take care: only returns probes on the same strand as the feature, f.
##  to return all probes, force all strands to be the same.
countMappableReadsInInterval <- function(features, UnMap, debug=FALSE, ...) {

    C <- sort(unique(as.character(seqnames(features))))
    
    ## Run parallel version.
    mcp <- mclapply(seq_along(C), countMappableReadsInInterval_foreachChrom, 
                    C=C, features=features, UnMap=UnMap, ...)

    ## Convert to a vector.
    FT <- rep(0,NROW(features))
    for(i in seq_along(C)) {
        indxF   <- which(as.character(seqnames(features)) == C[i])

        if(NROW(indxF) >0) {
            FT[indxF][mcp[[i]][["ord"]]] <- as.integer(mcp[[i]][["Difference"]])
        }
    }

    return(FT)
}


countMappableReadsInInterval_foreachChrom <- function(i, C, features, UnMap) {
    indxF   <- which(as.character(seqnames(features)) == C[i])

    if(NROW(indxF) >0) {
        # Order -- Make sure, b/c this is one of our main assumptions.  
        # Otherwise violated for DBTSS.
        Ford <- order(start(features[indxF,]))
        
        # Type coersions.
        FeatureStart    <- start(features[indxF,][Ford])
        FeatureEnd  <- end(features[indxF,][Ford])
        FeatureStr  <- as.character(strand(features[indxF,][Ford]))

        # Set dimensions.
        dim(FeatureStart)   <- c(NROW(FeatureStart), NCOL(FeatureStart))
        dim(FeatureEnd)     <- c(NROW(FeatureEnd),   NCOL(FeatureEnd))
        dim(FeatureStr)     <- c(NROW(FeatureStr),   NCOL(FeatureStr))
        
        ## Count start index.
        chr_indx <- which(UnMap[[1]][[1]] == C[i])
        CHRSIZE <- as.integer(UnMap[[1]][[2]][chr_indx])
        CHRSTART <- as.integer(0)
        if(chr_indx > 1) {  ## Running on 1:0 gives c(1, 0)
            CHRSTART <- as.integer( 
                sum(UnMap[[1]][[2]][
                    c(1:(chr_indx-1))
                ]) +1)
        }

        if(debug) {
            message(C[i],": Counting unMAQable regions.")
            message("CHRSIZE:", CHRSIZE, "CHRSTART:", CHRSTART)
        }

        ## Count unMAQable regions, and size of everything ... 
        nonmappable <- .Call("CountUnMAQableReads", FeatureStart, FeatureEnd, 
                UnMap[[2]], CHRSTART, CHRSIZE, PACKAGE = "groHMM")

        ## Adjust size of gene body.
        Difference <- (FeatureEnd - FeatureStart) - nonmappable + 1 
        ## Otherwise, get -1 for some.

        if(debug) {
            print(head(nonmappable))
            print(as.integer(head(Difference)))
        }

        #F[indxF][Ford] <- as.integer(Difference)
        return(list(Difference= Difference, ord= Ford))
    }
    return(integer(0))
}



 
#' readBed Returns a GenomicRanges object constrcuted from the specified bed
#' file.
#'
#' Bed file format is assumed to be either four column: seqnames, start, end,
#' strand columns; or six column: seqnames, start, end, name, score, and strand.
#' Three column format is also possible when there is no strand information.
#'
#' Any additional arguments availiable to read.table can be specified.
#'
#' @param file Path to the input file.
#' @param ... Extra argument passed to read.table
#' @return Returns GRanges object representing mapped reads.
#' @author Minho Chae and Charles G. Danko.
#' @examples
#' fileBed <- system.file(package = "rtracklayer", "tests", "test.bed")
#' readBed(fileBed, skip = 2, colClasses = c(rep(NA, 6), rep("NULL", 6)))
readBed <- function(file, ...) {
    df <- read.table(file, ...)
        if(NCOL(df) == 3) {
                colnames(df) <- c("seqnames", "start", "end")
                df <- cbind(df, strand=Rle("*", NROW(df)))
        }
        if(NCOL(df) == 4) colnames(df) <- c("seqnames", "start", "end", 
                                            "strand")
        if(NCOL(df) == 6) colnames(df) <- c("seqnames", "start", "end", 
                                            "name", "score", "strand")
        return( GRanges(seqnames = Rle(df$seqnames), ranges = 
                IRanges(df$start, df$end), strand = Rle(strand(df$strand))))
}

###########################################################################
##
##   Copyright 2013, 2014 Charles Danko and Minho Chae.
##
##   This program is part of the groHMM R package
##
##   groHMM is free software: you can redistribute it and/or modify it
##   under the terms of the GNU General Public License as published by
##   the Free Software Foundation, either version 3 of the License, or
##   (at your option) any later version.
##
##   This program is distributed in the hope that it will be useful, but
##   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
##   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
##   for more details.
##
##   You should have received a copy of the GNU General Public License along
##   with this program.  If not, see <http://www.gnu.org/licenses/>.
##
##########################################################################

#' Returns the number of cores.
#'
#' @param cores the number of cores, it is 1 in windows platform.
#' @examples
#' cores <- getCores(2L)
getCores <- function(cores)
{
    if (.Platform$OS.type =="windows")
        return(1L)
    cores
}

### R code from vignette source 'groHMM.Rnw'

###################################################
### code chunk number 1: style
###################################################
BiocStyle::latex(use.unsrturl=FALSE)
savewd = getwd()


###################################################
### code chunk number 2: install groHMM (eval = FALSE)
###################################################
## if (!requireNamespace("BiocManager", quietly=TRUE))
##     install.packages("BiocManager")
## BiocManager::install("groHMM")
## 


###################################################
### code chunk number 3: install packages (eval = FALSE)
###################################################
## BiocManager::install("GenomicFeatures")
## BiocManager::install("org.Hs.eg.db")
## BiocManager::install("edgeR")
## BiocManager::install("TxDb.Hsapiens.UCSC.hg19.knownGene")


###################################################
### code chunk number 4: use groHMM
###################################################
library(groHMM)
options(mc.cores=getCores(4))


###################################################
### code chunk number 5: load data
###################################################
S0mR1 <- as(readGAlignments(system.file("extdata", "S0mR1.bam",
        package="groHMM")), "GRanges")
S0mR2 <- as(readGAlignments(system.file("extdata", "S0mR1.bam",
        package="groHMM")), "GRanges") # Use R1 as R2
S40mR1 <- as(readGAlignments(system.file("extdata", "S40mR1.bam",
        package="groHMM")), "GRanges")
S40mR2 <- as(readGAlignments(system.file("extdata", "S40mR1.bam",
        package="groHMM")), "GRanges") # Use R1 as R2


###################################################
### code chunk number 6: combine replicates
###################################################
# Combine replicates
S0m <- c(S0mR1, S0mR2)
S40m <- c(S40mR1, S40mR2)


###################################################
### code chunk number 7: write wiggle files (eval = FALSE)
###################################################
## writeWiggle(reads=S0m, file="S0m_Plus.wig", fileType="wig", strand="+",
##         reverse=FALSE)
## writeWiggle(reads=S0m, file="S0m_Minus.wig", fileType="wig", strand="-",
##         reverse=TRUE)
## 
## # For BigWig file:
## # library(BSgenome.Hsapiens.UCSC.hg19)
## # si <- seqinfo(BSgenome.Hsapiens.UCSC.hg19)
## # writeWiggle(reads=S0m, file="S0m_Plus.wig", fileType="BigWig", strand="+",#
## #               reverse=FALSE, seqinfo=si)
## 
## # Normalized wiggle files
## expCounts <- mean(c(NROW(S0m), NROW(S40m)))
## writeWiggle(reads=S0m, file="S0m_Plus_Norm.wig", fileType="wig", strand="+",
##            normCounts=expCounts/NROW(S0m), reverse=FALSE)


###################################################
### code chunk number 8: transcript calling
###################################################
Sall <- sort(c(S0m, S40m))
# hmmResult <- detectTranscripts(Sall, LtProbB=-200, UTS=5,
#                   threshold=1)
# Load hmmResult from the saved previous run
load(system.file("extdata", "Vignette.RData", package="groHMM"))
txHMM <- hmmResult$transcripts


###################################################
### code chunk number 9: called transcripts
###################################################
head(txHMM)


###################################################
### code chunk number 10: use ucsc genes
###################################################
library(TxDb.Hsapiens.UCSC.hg19.knownGene)
kgdb <- TxDb.Hsapiens.UCSC.hg19.knownGene

library(GenomicFeatures)
# For refseq annotations:
# rgdb <- makeTxDbFromUCSC(genome="hg19", tablename="refGene")
# saveDb(hg19RGdb, file="hg19RefGene.sqlite")
# rgdb <- loadDb("hg19RefGene.sqlite")
kgChr7 <- transcripts(kgdb, filter=list(tx_chrom = "chr7"),
                      columns=c("gene_id", "tx_id", "tx_name"))
seqlevels(kgChr7) <- seqlevelsInUse(kgChr7)


###################################################
### code chunk number 11: make consensus annotations
###################################################
# Collapse overlapping annotations
kgConsensus <- makeConsensusAnnotations(kgChr7, keytype="gene_id",
        mc.cores=getOption("mc.cores"))
library(org.Hs.eg.db)
map <- select(org.Hs.eg.db,
    keys=unlist(mcols(kgConsensus)$gene_id),
        columns=c("SYMBOL"), keytype=c("ENTREZID"))
mcols(kgConsensus)$symbol <- map$SYMBOL
mcols(kgConsensus)$type <- "gene"


###################################################
### code chunk number 12: evaluation of transcript calling
###################################################
e <- evaluateHMMInAnnotations(txHMM, kgConsensus)
e$eval


###################################################
### code chunk number 13: tuning HMM
###################################################
tune <- data.frame(LtProbB=c(rep(-100,3), rep(-200,3), rep(-300,3)),
           UTS=rep(c(5,10,15), 3))

Fp <- windowAnalysis(Sall, strand="+", windowSize=50)
Fm <- windowAnalysis(Sall, strand="-", windowSize=50)

# evals <- mclapply(seq_len(9), function(x) {
#        hmm <- detectTranscripts(Fp=Fp, Fm=Fm, LtProbB=tune$LtProbB[x],
#                  UTS=tune$UTS[x])
#        e <- evaluateHMMInAnnotations(hmm$transcripts, kgConsensus)
#        e$eval
#        }, mc.cores=getOption("mc.cores"),  mc.silent=TRUE)
tune <- cbind(tune, do.call(rbind, evals))  # evals from the previous run


###################################################
### code chunk number 14: check tuning
###################################################
tune
which.min(tune$total)
which.min(tune$errorRate)


###################################################
### code chunk number 15: expressed genes
###################################################
getExpressedAnnotations <- function(features, reads) {
     fLimit <- limitToXkb(features)
     count <- countOverlaps(fLimit, reads)
     features <- features[count!=0,]
     return(features[(quantile(width(features), .05) < width(features))
    & (width(features) < quantile(width(features), .95)),])}
conExpressed <- getExpressedAnnotations(features=kgConsensus,reads=Sall)


###################################################
### code chunk number 16: fig2plot
###################################################
td <- getTxDensity(txHMM, conExpressed, mc.cores=getOption("mc.cores"))
u <- par("usr")
lines(c(u[1], 0, 0, 1000, 1000, u[2]), c(0,0,u[4]-.04,u[4]-.04,0,0),
      col="red")
legend("topright", lty=c(1,1), col=c(2,1), c("ideal", "groHMM"))
text(c(-500,500), c(.05,.5), c("FivePrimeFP", "TP"))
td


###################################################
### code chunk number 17: fig2
###################################################
td <- getTxDensity(txHMM, conExpressed, mc.cores=getOption("mc.cores"))
u <- par("usr")
lines(c(u[1], 0, 0, 1000, 1000, u[2]), c(0,0,u[4]-.04,u[4]-.04,0,0),
      col="red")
legend("topright", lty=c(1,1), col=c(2,1), c("ideal", "groHMM"))
text(c(-500,500), c(.05,.5), c("FivePrimeFP", "TP"))
td


###################################################
### code chunk number 18: Non-mammalian (eval = FALSE)
###################################################
## # mysql --user=genome --host=genome-mysql.cse.ucsc.edu ce10 -e \
## #  "select chrom, txStart, txEnd, name, exonCount, strand, name2 from refGene \
## #   where chrom not like chrom!='chrM' and cdsStart != cdsEnd" | tail -n +1 > refgene.bed
## 
## # G <- read.table("refgene.bed", header=TRUE, stringsAsFactors=FALSE, sep="\t")
## # ce <- GRanges(G$chrom, IRanges(G$txStart, G$txEnd), strand=G$strand, \
## #               access=G$name, gene_id=G$name2)
## # ceConsensus <- makeConsensusAnnotations(ce, keytype="gene_id", \
## #                mc.cores=getOption("mc.cores"))


###################################################
### code chunk number 19: repairing called transcripts
###################################################
bPlus <- breakTranscriptsOnGenes(txHMM, kgConsensus, strand="+")
bMinus <- breakTranscriptsOnGenes(txHMM, kgConsensus, strand="-")
txBroken <- c(bPlus, bMinus)
txFinal <- combineTranscripts(txBroken, kgConsensus)


###################################################
### code chunk number 20: plotTxDensity_final (eval = FALSE)
###################################################
## tdFinal <- getTxDensity(txFinal, conExpressed, mc.cores=getOption("mc.cores"))


###################################################
### code chunk number 21: DE for trascripts
###################################################
# For called transcripts
library(edgeR)
txLimit <- limitToXkb(txFinal)
ctS0mR1 <- countOverlaps(txLimit, S0mR1)
ctS0mR2 <- countOverlaps(txLimit, S0mR2)
ctS40mR1 <- countOverlaps(txLimit, S40mR1)
ctS40mR2 <- countOverlaps(txLimit, S40mR2)

pcounts <- as.matrix(data.frame(ctS0mR1, ctS0mR2, ctS40mR1, ctS40mR2))
group <- factor(c("S0m", "S0m", "S40m", "S40m"))
lib.size <- c(NROW(S0mR1), NROW(S0mR2), NROW(S40mR1), NROW(S40mR2))
d <- DGEList(counts=pcounts, lib.size=lib.size, group=group)
d <- estimateCommonDisp(d)
et <- exactTest(d)
de <- decideTests(et, p=0.001, adjust="fdr")
detags <- seq_len(NROW(d))[as.logical(de)]
# Number of transcripts regulated at 40m
cat("up: ",sum(de==1), " down: ", sum(de==-1), "\n")


###################################################
### code chunk number 22: fig3plot
###################################################
plotSmear(et, de.tags=detags)
# 2 fold up or down
abline(h = c(-1,1), col="blue")


###################################################
### code chunk number 23: fig3
###################################################
plotSmear(et, de.tags=detags)
# 2 fold up or down
abline(h = c(-1,1), col="blue")


###################################################
### code chunk number 24: DE for annotated genes
###################################################
# For ucsc knownGenes
kgChr7 <- transcripts(kgdb, filter <- list(tx_chrom = "chr7"),
                      columns=c("gene_id", "tx_id", "tx_name"))
map <- select(org.Hs.eg.db,
        keys=unique(unlist(mcols(kgChr7)$gene_id)),
        columns=c("SYMBOL"), keytype=c("ENTREZID"))
missing <- elementNROWS(mcols(kgChr7)[,"gene_id"]) == 0
kgChr7 <- kgChr7[!missing,]

inx <- match(unlist(mcols(kgChr7)$gene_id), map$ENTREZID)
mcols(kgChr7)$symbol <- map[inx,"SYMBOL"]

kgLimit <- limitToXkb(kgChr7)
ctS0mR1 <- countOverlaps(kgLimit, S0mR1)
ctS0mR2 <- countOverlaps(kgLimit, S0mR2)
ctS40mR1 <- countOverlaps(kgLimit, S40mR1)
ctS40mR2 <- countOverlaps(kgLimit, S40mR2)

counts <- as.matrix(data.frame(ctS0mR1, ctS0mR2, ctS40mR1, ctS40mR2))
group <- factor(c("S0m", "S0m", "S40m", "S40m"))
lib.size <- c(NROW(S0mR1), NROW(S0mR2), NROW(S40mR1), NROW(S40mR2))
d <- DGEList(counts=counts, lib.size=lib.size, group=group)

d <- estimateCommonDisp(d)
et <- exactTest(d)
de <- decideTests(et, p=0.001, adjust="fdr")
detags <- seq_len(NROW(d))[as.logical(de)]
symbols <- mcols(kgChr7)$symbol
# Number of unique genes regulated at 40m
cat("up: ", NROW(unique(symbols[de==1])), "\n")
cat("down: ", NROW(unique(symbols[de==-1])), "\n")
plotSmear(et, de.tags=detags)
abline(h = c(-1,1), col="blue")


###################################################
### code chunk number 25: metagene analysis
###################################################
upGenes <- kgChr7[de==1,]
expReads <- mean(c(NROW(S0m), NROW(S40m)))
# Metagene around TSS
mg0m <- runMetaGene(features=upGenes, reads=S0m, size=100,
                       normCounts=expReads/NROW(S0m), sampling=FALSE,
               mc.cores=getOption("mc.cores"))
mg40m <- runMetaGene(features=upGenes, reads=S40m, size=100,
                        normCounts=expReads/NROW(S40m), sampling=FALSE,
                mc.cores=getOption("mc.cores"))


###################################################
### code chunk number 26: plot metagene
###################################################
plotMetaGene <- function(POS=c(-10000:+9999), mg, MIN, MAX){
    plot(POS, mg$sense, col="red", type="h", xlim=c(-5000, 5000),
        ylim=c(floor(MIN),ceiling(MAX)), ylab="Read Density",
        xlab="Position (relative to TSS)")
     points(POS, (-1*rev(mg$antisense)), col="blue", type="h")
     abline(mean(mg$sense[5000:8000]), 0, lty="dotted")
}

MAX <- max(c(mg0m$sense, mg40m$sense))
MIN <- -1*max(c(mg0m$antisense, mg40m$antisense))
plotMetaGene(mg=mg0m, MIN=MIN, MAX=MAX)
plotMetaGene(mg=mg40m, MIN=MIN, MAX=MAX)


###################################################
### code chunk number 27: metagene figures
###################################################
png(filename="metagene0m.png")
plotMetaGene(mg=mg0m, MIN=MIN, MAX=MAX)
dev.off()
png(filename="metagene40m.png", width=480, height=480)
plotMetaGene(mg=mg40m, MIN=MIN, MAX=MAX)
dev.off()


###################################################
### code chunk number 28: save data for vignette (eval = FALSE)
###################################################
## save(hmmResult, evals, file="Vignette.RData")


###################################################
### code chunk number 29: sessionInfo
###################################################
toLatex(sessionInfo())


###################################################
### code chunk number 30: groHMM.Rnw:600-601
###################################################
setwd(savewd)

###########################################################################
##
##   Copyright 2013, 2014 Charles Danko and Minho Chae.
##
##   This program is part of the groHMM R package
##
##   groHMM is free software: you can redistribute it and/or modify it
##   under the terms of the GNU General Public License as published by
##   the Free Software Foundation, either version 3 of the License, or
##   (at your option) any later version.
##
##   This program is distributed in the hope that it will be useful, but
##   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
##   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
##   for more details.
##
##   You should have received a copy of the GNU General Public License along
##   with this program.  If not, see <http://www.gnu.org/licenses/>.
##
##########################################################################

#' windowAnalysis Returns a vector of integers representing the counts of
#' reads in a moving window.
#'
#' Supports parallel processing using mclapply in the 'parallel' package.
#' To change the number of processors, set the option 'mc.cores'.
#'
#' @param reads GenomicRanges object representing the position of reads
#' mapping in the genome.
#' @param strand Takes values of "+", "-", or "*".  "*" denotes collapsing
#' reads on both strands.  Default: "*".
#' @param windowSize Size of the moving window. Either windowSize or
#' stepSize must be specified.
#' @param stepSize The number of bp moved with each step.
#' @param chrom Chromosome for which to return data.
#' Default: returns all avaliable data.
#' @param limitPCRDups Counts only one read mapping to each start site.
#' NOTE: If set to TRUE, assumes that all reads are the same length
#' (don't use for paired-end data).  Default: FALSE.
#' @param ... Extra argument passed to mclapply
#' @return Returns a list object, each element of which represents a
#' chromosome.
#' @author Charles G. Danko and Minho Chae
#' @examples
#' S0mR1 <- as(readGAlignments(system.file("extdata", "S0mR1.bam",
#'      package="groHMM")), "GRanges")
#' ## Not run:
#' # Fp <- windowAnalysis(S0mR1, strand="+", windowSize=50)
windowAnalysis <- function(reads, strand="*", windowSize=stepSize, 
    stepSize=windowSize, chrom=NULL, limitPCRDups=FALSE, ...) {
    if (!(windowSize > 0 & (windowSize <= max(end(reads)))))
        stop("'windowSize' is out of range!")

    if (!(stepSize > 0 & (stepSize <= max(end(reads)))))
        stop("'stepSize' is out of range!")

    if (!is.null(chrom))  
        reads <- reads[seqnames(reads) == chrom,]

    readsList <- split(reads, seqnames(reads))
    if (limitPCRDups) {
        warning("Using limitPCRDups assumes all reads are the same size!  
            Don't use for paired end data!")
        readsList <- endoapply(readsList, function(x) {
            pStarts <- unique(start(x[strand(x) == "+",]))
            mEnds <- unique(end(x[strand(x) == "-",]))
            c(GRanges(seqnames(x)[1], IRanges(start=pStarts, width=1), 
                strand="+"), GRanges(seqnames(x)[1], 
                        IRanges(start=mEnds, width=1), strand="-"))

        })
    }

    # Change reads' strand 
    readsList <- endoapply(readsList, function(x) {
            if (strand == "*") 
                strand(x) <- "*"
            else
                x <- x[strand(x) == strand,]
            x 
    })

    H <- mclapply(readsList, function(x) {
            seqlevels(x) <- seqlevelsInUse(x)
            cov <- coverage(x)[[1]]
            to <- (length(cov) %/% windowSize)*windowSize
            starts <- seq(1, to, stepSize)
            vi <- Views(cov, start=starts, width=windowSize)
            Rle(viewSums(vi))
        }, ...) 
    
    
    return(H)

}

###########################################################################
##
##   Copyright 2013, 2014 Charles Danko and Minho Chae.
##
##   This program is part of the groHMM R package
##
##   groHMM is free software: you can redistribute it and/or modify it
##   under the terms of the GNU General Public License as published by
##   the Free Software Foundation, either version 3 of the License, or
##   (at your option) any later version.
##
##   This program is distributed in the hope that it will be useful, but
##   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
##   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
##   for more details.
##
##   You should have received a copy of the GNU General Public License along
##   with this program.  If not, see <http://www.gnu.org/licenses/>.
##
##########################################################################


#' detectTranscripts detects transcripts de novo using a two-state hidden
#' Markov model (HMM).
#'
#' Read counts can be specified as either a GRanges object (reads), or using a
#' fixed-step wiggle-format passed in a list (Fp and Fm).
#' Either reads or BOTH Fp and Fm must be specified.
#'
#' Supports parallel processing using mclapply in the 'parallel' package.
#' To change the number of processors set the option 'mc.cores'.
#'
#' Reference: Hah N, Danko CG, Core L, Waterfall JJ, Siepel A, Lis JT,
#' Kraus WL. A rapid, extensive, and transient transcriptional response to
#' estrogen signaling in breast cancer cells. Cell. 2011 May 13;145(4):622-34.
#' doi: 10.1016/j.cell.2011.03.042.
#'
#' @param reads A GRanges object representing a set of mapped reads.
#' @param Fp Wiggle-formatted read counts on "+" strand. Optionally, Fp and Fm
#' represent list() filled with a vector of counts for each chromosome.
#' Can detect transcripts starting from a fixed-step wiggle.
#' @param Fm Wiggle-formatted read counts on "-" strand.
#' @param LtProbA Log probability of t... .  Default: -5. One of these is just
#' an initialization, and the final value is set by EM.  The other is a holdout
#' parameter.
#' @param LtProbB Log probability of t... .  Default: -200.
#' @param UTS Varience in read counts of the untranscribed sequence.
#' Default: 5.
#' @param size Log probability of t... .  Default: -5.
#' @param threshold Threshold change in total likelihood, below which EM exits.
#' @param debug If set to TRUE, provides additional print options.
#' Default: FALSE
#' @param ... Extra argument passed to mclapply
#' @return Returns a list of emisParams, trnasParams, viterbiStates, and
#' transcripts.  The transcript element is a GRanges object representing the
#' predicted genomic coordinates of transcripts on both the + and - strand.
#' @author Charles G. Danko and Minho Chae
#' @examples
#' S0mR1 <- as(readGAlignments(system.file("extdata", "S0mR1.bam",
#'                package="groHMM")), "GRanges")
#' ## Not run:
#' # hmmResult <- detectTranscripts(S0mR1, LtProbB=-200, UTS=5, threshold=1)
#' # txHMM <- hmmResult$transcripts
## CGD: TODO: Test switch over to gamma, rather than dGamma?!

detectTranscripts <- function(reads=NULL, Fp=NULL, Fm=NULL, LtProbA=-5, 
    LtProbB=-200, UTS=5, size=50, threshold=0.1, debug=TRUE, ...) {

    stopifnot(!is.null(reads)|(!is.null(Fp) & !is.null(Fm)))

    ## Setup/Window Analysis/Casting.
    epsilon <- 0.001

    ## Allow equilavent form of Fp and Fm to be spcified in the function 
    ## automatically.
    if(is.null(Fp) & is.null(Fm)) { 
     Fp <- windowAnalysis(reads=reads, strand="+", windowSize=size, ...)
     Fm <- windowAnalysis(reads=reads, strand="-", windowSize=size, ...)
    }
    
    nFp <- NROW(Fp)
    nFm <- NROW(Fm)
    CHRp <- as.character(names(Fp))
    CHRm <- as.character(names(Fm))

    size <- as.integer(size)
    ANS <- NULL

    ## Set up initial HMM variables.
    HMM <- list()
    HMM$nstates <- as.integer(2)
    HMM$ePrDist <- c("dgamma", "dgamma") 
    ## CGD: 3-3-13: Still legacy. Switch to integrating gamma between read 
    ## and read+1

    HMM$iProb <- as.double(log(c(1.0,0.0)))
                                    ## Non-transcribed,  transcribed.
    HMM$ePrVars <- as.list(data.frame(c(UTS, 1/UTS, -1), c(0.5, 10, -1)))
    HMM$tProb <- as.list(data.frame(c(log(1-exp(LtProbA)), LtProbA), 
        c(LtProbB, log(1-exp(LtProbB))) ))

    ## Cast counts to a real, and combine +/- strand into one list variable.  
    ##  Treat like separate training sequences (they really are).
    FT <- list()    # MHC; 7/2/2014, bioconductor complains F thinking as False
    for(i in 1:nFp) FT[[i]]<- as.double(Fp[[i]]+1) 
    ## CGD: 3-3-13: Still legacy.  Switch to integrating gamma between read and 
    ## read+1

    for(i in 1:nFm) FT[[i+nFp]] <- as.double(Fm[[i]]+1) 
    ## CGD: 3-3-13: Still legacy.  Switch to integrating gamma between read and
    ## read+1

    ## In case the above command copies, rather than points ... 
    ## free unused memory.
    remove(Fp)
    remove(Fm)

    ## Run EM algorithm.
    BWem <- .Call("RBaumWelchEM", HMM$nstates, FT, as.integer(1),
                HMM$ePrDist, HMM$ePrVars, HMM$tProb, HMM$iProb, 
                as.double(threshold), c(TRUE, FALSE), c(FALSE, TRUE), 
                as.integer(1), TRUE, PACKAGE="groHMM")
               # Update Transitions, Emissions.

    ## Translate these into transcript positions.
    for(i in seq_along(CHRp)) {
        ans <- .Call("getTranscriptPositions", as.double(BWem[[3]][[i]]), 
                    as.double(0.5), size, PACKAGE="groHMM")
        Nrep <- NROW(ans$Start)
        # ANS <- rbind(ANS, data.frame(chrom =rep(CHRp[i], Nrep), 
        #           chromStart =ans$Start, chromEnd =ans$End, 
        #   name =rep("N", Nrep), score =rep("1", Nrep), strand =rep("+", 
        #           Nrep)))
            ANS <- rbind(ANS, data.frame(chrom =rep(CHRp[i], Nrep), 
                start=ans$Start, end =ans$End, strand =rep("+", Nrep)))
    }

    for(i in seq_along(CHRm)) {
        ans <- .Call("getTranscriptPositions", as.double(BWem[[3]][[i+nFp]]), 
                    as.double(0.5), size, PACKAGE="groHMM")
        Nrep <- NROW(ans$Start)
        # ANS <- rbind(ANS, data.frame(chrom =rep(CHRm[i], NROW(ans$Start)), 
        #           chromStart =ans$Start, chromEnd =ans$End, 
        #           name =rep("N", Nrep), score =rep("1", Nrep), 
        #           strand =rep("-", Nrep)))
            ANS <- rbind(ANS, data.frame(chrom =rep(CHRm[i], NROW(ans$Start)), 
                    start=ans$Start, end=ans$End, strand =rep("-", Nrep)))
    }

    #BWem[[4]] <- ANS
    #names(BWem) <- c("EmisParams", "TransParams", "ViterbiStates", 
    #                   "Transcripts")

    BWem[[4]] <- GRanges(seqnames = Rle(ANS$chrom), 
                    ranges = IRanges(ANS$start, ANS$end-1), 
                    strand = Rle(strand(ANS$strand)), type=Rle("tx",NROW(ANS)), 
                    ID=paste(ANS$chrom, "_", ANS$start, ANS$strand, sep=""))
        names(BWem) <- c("emisParams", "transParams", "viterbiStates", 
                            "transcripts")

    if(debug) {
        print(BWem[[1]])
        print(BWem[[2]])
    }

    return(BWem)
}

###########################################################################
##
##   Copyright 2013, 2014 Charles Danko and Minho Chae.
##
##   This program is part of the groHMM R package
##
##   groHMM is free software: you can redistribute it and/or modify it
##   under the terms of the GNU General Public License as published by
##   the Free Software Foundation, either version 3 of the License, or
##   (at your option) any later version.
##
##   This program is distributed in the hope that it will be useful, but
##   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
##   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
##   for more details.
##
##   You should have received a copy of the GNU General Public License along
##   with this program.  If not, see <http://www.gnu.org/licenses/>.
##
##########################################################################
plot2Ranges <- function(tr, gr, main = "NA", col = "black", sep = 0.5, ...) {
    height <- 1
    xlim <- c(min(min(start(tr), min(start(gr)))), max(max(end(tr)), 
                max(end(gr)))) 

    plot.new()
    ybottom <- seq(1,length(tr)+length(gr))
    plot.window(xlim, c(0,max(ybottom+height)))

    if (length(tr)>0) {
        tbottom <- seq(1,length(tr))
        rect(start(tr)-sep, tbottom, end(tr)+sep, tbottom + height-sep, 
                col = "gray", ...)
    }

    bSymbol <- FALSE
    if ("symbol" %in% colnames(elementMetadata(gr)))
    bSymbol <- TRUE

    if (length(gr) >0) {
        gbottom <- seq(1,length(gr))
        rect(start(gr)-sep, gbottom, end(gr)+sep, gbottom + height-sep,
            col = rgb(red=255, green=0, blue=0, alpha=150, maxColorValue=255))
        if (bSymbol)
            text((start(gr) + end(gr))/2, gbottom + 0.2, 
                elementMetadata(gr)$symbol) 
    }
    title(main)
    axis(1)
}
    


#
# Break an Interval by break points with minimum gap 
#------------------------------------------------------------------------------
breakInterval <- function(gr, brPos, gap=5, strand="+") {
    result <- rep(gr, length(brPos)+1)
    if (strand == "+") {
        for(i in seq_along(brPos)) { 
            end(result[i,]) <- brPos[i] - gap   
            start(result[i+1,]) <- brPos[i]
        }
    } else {
        for(i in seq_along(brPos)) { 
            end(result[i,]) <- brPos[i] 
            start(result[i+1,]) <- brPos[i] + gap
        }
    }
    return(result)
}


#' breakTranscriptsOnGenes Breaks transcripts on genes
#'
#' Breaks transcripts when they are overlapped with multiple well annotated
#' genes.
#'
#' @param tx GRanges of transcripts.
#' @param annox GRanges of non-overlapping annotations for reference.
#' @param strand Takes "+" or "-" Default: "+"
#' @param geneSize Numeric. Minimum gene size in annox to be used as reference.
#' Default: 5000
#' @param threshold Numeric. Ratio of overlapped region relative to a gene
#' width.
#' Transcripts only greater than this threshold are subjected to be broken.
#' Default: 0.8
#' @param gap Numeric.  Gap (bp) between broken transcripts.  Default: 5
#' @param plot Logical.  If set to TRUE, show each step in a plot.
#' Default: FALSE
#' @author Minho Chae and Charles G. Danko
#' @return Returns GRanges object of broken transcripts.
#' @examples
#' tx <- GRanges("chr7", IRanges(1000, 30000), strand="+")
#' annox <- GRanges("chr7", IRanges(start=c(1000, 20000),
#'              width=c(10000,10000)), strand="+")
#' bPlus <- breakTranscriptsOnGenes(tx, annox, strand="+")
breakTranscriptsOnGenes <- function(tx, annox, strand="+", geneSize=5000, 
    threshold=0.8, gap=5, plot=FALSE) {
    tx <- tx[strand(tx) == strand,]
    annox <- annox[strand(annox) == strand,]
    mcols(tx)$status <- "NA"

    annox <- annox[width(annox) > geneSize,]
    ol <- findOverlaps(tx, annox)

    ol.df <- data.frame(trans=queryHits(ol), gene=subjectHits(ol), 
        ratio=width(pintersect(tx[queryHits(ol),], annox[subjectHits(ol),]))
        /width(annox[subjectHits(ol),]))

    # Keep only major overlappings
    ol.df <- ol.df[ol.df$ratio > threshold,]

    # Keep only multiple genes on a transcript cases
    dupTrans <- unique(ol.df$trans[duplicated(ol.df$trans)])
    ol.df <- ol.df[ol.df$trans %in% dupTrans,]

    ol.tab <- table(ol.df$trans)
    ol.tabCS <- cumsum(ol.tab)

    bT <- NULL

    for (i in seq_along(ol.tab)) {
        txNo <- as.integer(names(ol.tab[i]))
        aNo <- ol.df$gene[(ol.tabCS[i]-ol.tab[i]+1):ol.tabCS[i]]

        if (strand == "+") {
            # No need of break position for the first annotation
            brPos <- start(annox[aNo[-1],])        
        } else {
            # No need of break position for the last annotation
            brPos <- end(annox[aNo[-length(aNo)],])  
        }

        frags <- breakInterval(tx[txNo,], brPos=brPos, gap=gap, strand=strand)

        if(is.null(bT)) {
            bT <- frags 
        } else {
            bT <- c(bT, frags) 
        }
        if (plot) {
            par(mfrow=c(2,1))
            plot2Ranges(tx[txNo,], annox[aNo,], main="Before")
            plot2Ranges(frags, annox[aNo,], main="After")
            Sys.sleep(5)
        }
    }

    mcols(bT)$status <- "broken"
    mcols(bT)$ID <- paste(seqnames(bT), "_", start(bT), strand(bT), sep="")
    okTrans <- setdiff(1:length(tx), dupTrans)
    all <- c(tx[okTrans,], bT)
    cat(length(unique(ol.df$trans)), " transcripts are broken into ", 
        length(bT), "\n")

    return(all[order(as.character(seqnames(all)), start(all)),])
}

#' combineTranscripts Combines transnscipts.
#'
#' Combines transcripts  that are within the same gene annotation, combining
#' smaller transcripts for genes
#'  with low regulation into a single transcript representing the gene.
#'
#' @param tx GRanges of transcripts.
#' @param annox GRanges of non-overlapping annotations for reference.
#' @param geneSize Numeric. Minimum gene size in annotations to be used as
#' reference.
#' Default: 1000
#' @param threshold Numeric. Ratio of overlapped region relative to transcript
#' width.
#' Transcripts only greater than this threshold are subjected to be combined.
#' Default: 0.8
#' @param plot Logical.  If set to TRUE, show easch step in a plot.
#' Default: FALSE
#' @return Returns GRanges object of combined transcripts.
#' @author Minho Chae and Charles G. Danko
#' @examples
#' tx <- GRanges("chr7", IRanges(start=c(1000, 20000), width=c(10000,10000)),
#' strand="+")
#' annox <- GRanges("chr7", IRanges(1000, 30000), strand="+")
#' combined <- combineTranscripts(tx, annox)
combineTranscripts <- function(tx, annox, geneSize=1000, threshold=0.8, 
    plot=FALSE) {
    annox <- annox[width(annox) > geneSize,]
    ol <- findOverlaps(tx, annox)
    ol.df <- data.frame(trans=queryHits(ol), gene=subjectHits(ol),
        ratio=width(pintersect(tx[queryHits(ol),], annox[subjectHits(ol),]))
                /width(tx[queryHits(ol),]))

    # Keep only major overlappings
    ol.df <- ol.df[ol.df$ratio > threshold,]

    # Keep only multiple transcripts on a gene cases
    dupGenes <- ol.df$gene[which(duplicated(ol.df$gene))]
    ol.df <- ol.df[ol.df$gene %in% dupGenes,]

    uniqGene <- unique(ol.df$gene)
    N <- length(uniqGene)
    cT <- GRanges(seqnames=Rle(rep("chr1", N)),
        ranges = IRanges(rep(1, N), rep(max(end(tx)), N)),
        strand = rep("+", N),
        type = Rle(rep("tx", N)))
    seqlevels(cT) <- seqlevels(tx)

    for (i in seq_along(uniqGene)) {
        block <- ol.df[uniqGene[i]==ol.df$gene,]
        strand(cT[i,]) <- strand(tx[block[1,"trans"],])
        seqnames(cT[i,]) <- seqnames(tx[block[1,"trans"],])
        start(cT[i,]) <- start(tx[block[1,"trans"],])
        end(cT[i,]) <- end(tx[block[NROW(block),"trans"],])
        if (plot) { # plot before and after the comine
            #b4combine <- c(gr[uniqGene[i],], tr[ol.df[ol.df$gene == 
            # uniqGene[i], "trans"],-2])
            #af.combine <- c(gr[uniqGene[i],], cT[i,])
            par(mfrow=c(2,1))
            plot2Ranges(tx[ol.df[ol.df$gene == uniqGene[i], "trans"],], 
                annox[uniqGene[i],], main="Before")
            plot2Ranges(cT[i,], annox[uniqGene[i],], main="After")
            Sys.sleep(5)
        }
    }

    cat(NROW(ol.df), " transcripts are combined to ", NROW(cT), "\n")
    mcols(cT)$ID <- paste(seqnames(cT), "_", start(cT), strand(cT), sep="")
    mcols(cT)$status <- "combined"

    okTrans <- setdiff(seq_along(tx), ol.df$trans)
    all <- c(tx[okTrans,], cT)
    return(all[order(as.character(seqnames(all)), start(all)),])
}

###########################################################################
##
##   Copyright 2009, 2010, 2011 Charles Danko and Minho Chae.
##
##   This program is part of the groHMM R package
##
##   groHMM is free software: you can redistribute it and/or modify it 
##   under the terms of the GNU General Public License as published by 
##   the Free Software Foundation, either version 3 of the License, or  
##   (at your option) any later version.
##
##   This program is distributed in the hope that it will be useful, but 
##   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
##   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
##   for more details.
##
##   You should have received a copy of the GNU General Public License along 
##   with this program.  If not, see <http://www.gnu.org/licenses/>.
##
##########################################################################


#####################################################
##
## tlsFits.R -- Total least squares fits for several functional forms, 
## including:
##  -- Linear; done using Demming regression (tlsDemming), and svd (tlsSvd).
##  -- LOESS; hacked approximation done by transforming data.  Rotats data so 
##      that the LOESS fit approximates a given angle (by default 45 degrees; 
##      assumes varience of two data types is equal). 
##
##
## WARNING -- FUNCTIONS CURRENTLY EXPERIMENTAL!! USE WITH EXTREME CATION!!
##
## WARNING -- CURRENTLY DEMMING AND SVD METHODS FOR FITTING TOTAL LEAST 
##          SQUARES DO NOT AGREE!!  SEE BELOW FOR MORE DISCUSSION.
##
#####################################################


#' A 'total least squares'-like hack for LOESS.  Works by rotating points
#' 45 degrees, fitting LOESS, and rotating back.
#'
#' @param x X values.
#' @param y Y values.
#' @param theta Amount to rotate, sets the ratio of variences that are assumed
#' by the hack.  Default: -pi/4 radians (45 degrees) for orthogonal regression.
#' @param span The LOESS span parameter.  Default: 1
#' @return List of input values and LOESS predictions.
#' @author Charles G. Danko
## Transform and fit LOESS!
##
## By transforming X and Y by 45 degrees, we can fit using LOESS and it's
## essentially "erros in variables" LOESS.
##
## Returns x and y values giving information on the fit...
tlsLoess <- function(x, y, theta= -pi/4, span= 1) {
 mx <- mean(x)
 my <- mean(y)
 x <- x-mx
 y <- y-my
 tx <- x*cos(theta)-y*sin(theta)
 ty <- x*sin(theta)+y*cos(theta)
 
 lo <- loess(ty~tx, family="gaussian", span = span)
 yp_loess  <- predict(lo,data.frame(x=tx))

 theta <- -theta
 x <- tx*cos(theta)-yp_loess*sin(theta)
 y <- tx*sin(theta)+yp_loess*cos(theta)
 x <- x+mx
 y <- y+my
 
 ord <- order(x)
 x <- x[ord]
 y <- y[ord]
 
 retVar <- list()
 retVar$x <- x
 retVar$y <- y
 
 return(retVar)
}

#' A 'total least squares' implementation using singular value demposition.
#'
#' @param x X values.
#' @param y Y values.
#' @return Parameters for the linear model Y~a*X+e.
#' @author Charles G. Danko
## linear total least squares by SVD.
## Test: 
## A <- as.matrix(rbind(c(1,1,1), c(2,-1,2), c(-1,4,3), c(4,2,1), c(3,-3,4)))
## b <- c(1,2,-1,4,8)
## 
tlsSvd <- function(x,y) {
 n <- NCOL(x)
 Cmat <- as.matrix(data.frame(x, y))
 s <- svd(Cmat)
 VAB <- s$v[c(1:n),(1+n):NCOL(s$v)]
 VBB <- s$v[(1+n):NROW(s$v),(1+n):NCOL(s$v)]
 X = -VAB/VBB
 int <- mean(y) - X * mean(x)
 return(c(int,X))
}

#' A 'total least squares' implementation using demming regression.
#'
#' @param x X values.
#' @param y Y values.
#' @param d Ratio of variences. Default: 1, for orthogonal regression.
#' @return Parameters for the linear model.
#' @author Charles G. Danko
## linear total least squares by Deming regression.
tlsDeming <- function(x,y,d=1) {
 sxx <- 1/(NROW(x)-1) * sum((x-mean(x))^2)
 sxy <- 1/(NROW(x)-1) * sum((x-mean(x))*(y-mean(y)))
 syy <- 1/(NROW(y)-1) * sum((y-mean(y))^2)
 X <- (syy-d*sxx+sqrt((syy-d*sxx)^2+4*d*(sxy^2)))/(2*sxy)
 int <- mean(y) - X * mean(x)
 return(c(int,X))
}

###########################################################################
##
##   Copyright 2013, 2014 Charles Danko and Minho Chae.
##
##   This program is part of the groHMM R package
##
##   groHMM is free software: you can redistribute it and/or modify it
##   under the terms of the GNU General Public License as published by
##   the Free Software Foundation, either version 3 of the License, or
##   (at your option) any later version.
##
##   This program is distributed in the hope that it will be useful, but
##   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
##   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
##   for more details.
##
##   You should have received a copy of the GNU General Public License along
##   with this program.  If not, see <http://www.gnu.org/licenses/>.
##
##########################################################################




#' makeConsensusAnnotations Makes a consensus annotation
#'
#' Makes a non-overlapping consensus annotation.  Gene annotations are often
#' overalpping due to #' multiple isoforms for a gene.
#' In consensus annotation, isoforms are first reduced so that only
#' redundant intervals are used to represent a genomic interval for a gene,
#' i.e., a gene id.
#' Remaining unresolved annotations are further reduced by truncating 3'
#' end of annotations.
#'
#' Supports parallel processing using mclapply in the 'parallel' package.
#' To change the number of processors, use the argument 'mc.cores'.
#'
#' @param ar GRanges of annotations to be collapsed.
#' @param minGap Minimun gap between overlapped annotations after truncated.
#' Default: 1L
#' @param minWidth Minimun width of consensus annotations. Default: 1000L
#' @param ... Extra argument passed to mclapply.
#' @return Returns GRanges object of annotations.
#' @author Minho Chae
#' @examples
#' library(TxDb.Hsapiens.UCSC.hg19.knownGene)
#' txdb <- TxDb.Hsapiens.UCSC.hg19.knownGene
#' tx <- transcripts(txdb, columns = c("gene_id", "tx_id", "tx_name"),
#'                   filter = list(tx_chrom = "chr7"))
#' tx <- tx[grep("random", as.character(seqnames(tx)), invert=TRUE), ]
#' ## Not run:
#' # ca <- makeConsensusAnnotations(tx)
makeConsensusAnnotations <- function(ar, minGap=1L, minWidth=1000L, ...) {
    # check missing gene_id
    missing <- elementNROWS(mcols(ar)[,"gene_id"]) == 0 
    if (any(missing)) {
        ar <- ar[!missing,]
        warning(sum(missing), " ranges do not have gene_id and they are 
            dropped")
    }

    many <- elementNROWS(mcols(ar)[,"gene_id"]) > 1 
    if (any(many)) {
        ar <- ar[!many,]
        warning(sum(many), " ranges have multiple gene_id and they are 
            dropped")
    }

    ar_list <- split(ar, unlist(mcols(ar)[,"gene_id"]))
    singles <- unlist(ar_list[elementNROWS(ar_list) == 1])
    isoforms <- ar_list[elementNROWS(ar_list) > 1]
    
    message("Reduce isoforms(", length(isoforms),") ... ", appendLF=FALSE)
    isoforms <- GRangesList(mclapply(isoforms, function(x) {
        # For mixed strands or chrom, choose the longest    
        if ((length(seqlevelsInUse(x)) > 1) || 
                (length(unique(strand(x))) > 1)) {
            result <- x[which.max(width(x)), "gene_id"]
        } else {
            dx <- disjoin(x)
            mcols(dx)$gene_id <- mcols(x)$gene_id[1]
            olcnt <- countOverlaps(dx, x)

            multi <- dx[olcnt > 1]    # Use the disjoint ranges 
                                      # covered more than once
            if (length(multi) == 0) { # For non-overlapping isoforms, 
                                      # choose the longest
                result <- x[which.max(width(x)), "gene_id"]
            } else if (length(multi) == 1) {
                result <- multi
            } else {
                reduced <- reduce(multi)
                if (length(reduced) == 1) 
                    result <- reduced
                else (length(reduced) > 1) 
                    result <- reduced[which.max(width(reduced)),]
                
            }
            mcols(result)$gene_id <- mcols(x)$gene_id[1]
        }
        return(result)
    }, mc.cores=getCores(10)))
    isoforms <- unlist(isoforms)
    message("OK")

    # Check redundancy 
    isoforms <- removeRedundant(isoforms)
    singles <- removeRedundant(singles)

    o <- findOverlaps(singles, isoforms, type="equal")
    if(length(o) != 0)
        singles <- singles[-queryHits(o),]

    o <- findOverlaps(singles, isoforms, type="within")
    if(length(o) != 0)
        singles <- singles[-queryHits(o),]

    o <- findOverlaps(isoforms, singles, type="within")
    if(length(o) != 0)
        isoforms <- isoforms[-queryHits(o),]

    noiso <- sort(c(isoforms, singles[,"gene_id"])) 
    message("Truncate overlapped ranges ... ", appendLF=FALSE)      
    # with different gene_ids
    while(!isDisjoint(noiso)) { 
        ol <- findOverlaps(noiso, drop.self=TRUE, drop.redundant=TRUE)
        ol_gr <- GRangesList(lapply(1:length(ol), function(x) {
                        sort(c(noiso[queryHits(ol)[x]], 
                                noiso[subjectHits(ol)[x]])) 
                }))

        # Truncate 3' end
        ol_gr <- unlist(endoapply(ol_gr, function(x) {
            if (as.character(strand(x[1,])) == "+") {
                end(x[1,]) <- start(x[2,]) - minGap     
                # first range's end is truncated
            } else {
                start(x[2,]) <- end(x[1,]) + minGap     
                # sencond range's end is truncated
            }
            x
        }))

        # Remove any ranges with duplicated names since they already adujsted 
        # in the previous call
        ol_gr <- ol_gr[!duplicated(names(ol_gr)),]
        
        noiso <- noiso[-unique(c(queryHits(ol), subjectHits(ol))),] 
        # update noiso
        noiso <- c(noiso, ol_gr)
    }
    message("OK")

    noiso <- noiso[width(noiso) >= minWidth,]
    return(sort(noiso))
}

removeRedundant <- function(annox) {
    o <- findOverlaps(annox, drop.self=TRUE, type="equal", 
            drop.redundant=TRUE)
    if(length(o) != 0)
        annox <- annox[-subjectHits(o),]

    o <- findOverlaps(annox, drop.self=TRUE, type="within", 
            drop.redundant=TRUE)
    if(length(o) != 0)
        annox <- annox[-queryHits(o),]

    return(annox)
}

###########################################################################
##
##   Copyright 2013, 2014 Charles Danko and Minho Chae.
##
##   This program is part of the groHMM R package
##
##   groHMM is free software: you can redistribute it and/or modify it
##   under the terms of the GNU General Public License as published by
##   the Free Software Foundation, either version 3 of the License, or
##   (at your option) any later version.
##
##   This program is distributed in the hope that it will be useful, but
##   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
##   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
##   for more details.
##
##   You should have received a copy of the GNU General Public License along
##   with this program.  If not, see <http://www.gnu.org/licenses/>.
##
##########################################################################


#' Returns a histogram of the number of reads in each section of a moving
#' window centered on a certain feature.
#'
#' Supports parallel processing using mclapply in the 'parallel' package.
#' To change the number of processors, set the option 'mc.cores'.
#'
#' @param features A GRanges object representing a set of genomic coordinates.
#' The meta-plot will be centered on the transcription start site (TSS)
#' @param reads A GRanges object representing a set of mapped reads.
#' Instead of 'reads', 'plusCVG' and 'minusCVG' can be used  Default: NULL
#' @param plusCVG An IntegerRangesList object for reads with '+' strand.
#' @param minusCVG An IntegerRangesList object for reads with '-' strand.
#' @param size The size of the moving window.
#' @param up Distance upstream of each features to align and histogram.
#' Default: 10 kb.
#' @param down Distance downstream of each features to align and histogram.
#' If NULL, same as up. Default: NULL.
#' @param ... Extra argument passed to mclapply
#' @return Returns a integer-Rle representing the 'typical' signal
#' centered on a point of interest.
#' @author Charles G. Danko and Minho Chae
#' @examples
#' features <- GRanges("chr7", IRanges(1000, 1000), strand="+")
#' reads <- GRanges("chr7", IRanges(start=c(1000:1004, 1100),
#'  width=rep(1, 6)), strand="+")
#' mg <- metaGene(features, reads, size=4, up=10)
metaGene <- function(features, reads=NULL, plusCVG=NULL, minusCVG=NULL, 
    size=100L, up=10000L, down=NULL, ...) {
    seqlevels(features) <- seqlevelsInUse(features)
    # Check 'reads'
    if (is.null(reads)) {
        if (is.null(plusCVG) || is.null(minusCVG))
            stop("Either 'reads' or 'plusCVG' and 'minusCVG' must be used")
    } else {
        seqlevels(reads) <- seqlevelsInUse(reads)
        plusCVG <- coverage(reads[strand(reads)=="+",])
        minusCVG <- coverage(reads[strand(reads)=="-",])
    }
    if (is.null(down)) down <- up

    featureList <- split(features, seqnames(features))

    H <- mclapply(seqlevels(features), metaGene_foreachChrom, 
        featureList=featureList, plusCVG=plusCVG, minusCVG=minusCVG, 
        size=size, up=up, down=down, ...)
    M <- sapply(seq_len(length(H)), function(x) as.integer(H[[x]]))

    return(Rle(apply(M, 1, sum)))
}


metaGene_foreachChrom <- function(chrom, featureList, plusCVG, minusCVG, 
    size, up, down) {
        f <- featureList[[chrom]]

        pCVG <- plusCVG[[chrom]]
        mCVG <- minusCVG[[chrom]]
        offset <- floor(size/2L)

        pro <- promoters(f, upstream=up+offset, downstream=(down+offset-1L))

        M <- sapply(1:length(pro), function(x) {
                if (as.character(strand(pro)[x]) == "+")
                    as.integer(runsum(pCVG[start(pro)[x]:end(pro)[x]], 
                        k=size))
                else
                    as.integer(rev(runsum(mCVG[start(pro)[x]:end(pro)[x]], 
                        k=size)))
            })
        return(Rle(apply(M, 1, sum)))
}


#' Runs metagene analysis for sense and antisense direction.
#'
#' Supports parallel processing using mclapply in the 'parallel' package.
#' To change the number of processors, set the option 'mc.cores'.
#'
#' @param features GRanges A GRanges object representing a set of genomic
#' coordinates, i.e., set of genes.
#' @param reads GRanges of reads.
#' @param anchorType Either 'TSS' or 'TTS'.  Metagene will be centered on the
#' transcription start site(TSS) or transcription termination site(TTS).
#' Default: TSS.
#' @param size Numeric.  The size of the moving window. Default: 100L
#' @param normCounts Numeric.  Normalization vector such as average reads.
#' Default: 1L
#' @param up Numeric. Distance upstream of each feature to align and histogram.
#' Default: 1 kb
#' @param down Numeric. Distance downstream of each feature to align and
#' histogram.  If NULL, down is same as up. Default: NULL
#' @param sampling Logical.  If TRUE, subsampling of Metagene is used.
#' Default: FALSE
#' @param nSampling Numeric. Number of subsampling.  Default: 1000L
#' @param samplingRatio Numeric. Ratio of sampling for features.  Default: 0.1
#' @param ... Extra argument passed to mclapply.
#' @return A list of integer-Rle for sense and antisene.
#' @author Minho Chae
#' @examples
#' features <- GRanges("chr7", IRanges(start=1000:1001, width=rep(1,2)),
#'  strand=c("+", "-"))
#' reads <- GRanges("chr7", IRanges(start=c(1000:1003, 1100:1101),
#'  width=rep(1, 6)), strand=rep(c("+","-"), 3))
#' ## Not run:
#' # mg <- runMetaGene(features, reads, size=4, up=10)
runMetaGene <- function(features, reads, anchorType="TSS", size=100L, 
    normCounts=1L, up=10000L, down=NULL, sampling=FALSE, nSampling=1000L,
    samplingRatio=0.1, ...) {
    # Check 'anchorType'
    if (!anchorType %in% c("TSS", "TTS")) {
        stop("'anchorType' must be either 'TSS' or 'TTS'")
    }

    if (anchorType == "TSS") {
        f <- resize(features, width=1L, fix="start")
    } else if (anchorType == "TTS") {
        f <- resize(features, width=1L, fix="end")
    }

    if (is.null(down)) down <- up

    fRev <- f
    strand(fRev) <- rev(strand(f))

    plusCVG <- coverage(reads[strand(reads)=="+",])
    minusCVG <- coverage(reads[strand(reads)=="-",])
    
    message("sense ... ", appendLF=FALSE) 
    if (sampling) { 
        sense <- samplingMetaGene(features=f, plusCVG=plusCVG, 
            minusCVG=minusCVG, size=size, up=up, down=down, 
            nSampling=nSampling, samplingRatio=samplingRatio, ...) 
    } else {
        sense <- metaGene(features=f, plusCVG=plusCVG, minusCVG=minusCVG, 
                    size=size, up=up, down=down, ...)
        sense <- sense/length(features)
    }
    message("OK")

    message("antisense ... ", appendLF=FALSE)
    if (sampling) {
        antisense <- samplingMetaGene(features=fRev, plusCVG=plusCVG, 
            minusCVG=minusCVG, size=size, up=up, down=down,
            nSampling=nSampling, samplingRatio=samplingRatio, ...)
    } else {
        antisense <- metaGene(features=fRev, plusCVG=plusCVG, minusCVG=minusCVG,
            size=size, up=up, down=down, ...)
        antisense <- antisense/length(features)
    }
    message("OK")
    
    sense <- sense*normCounts
    antisense <- antisense*normCounts
    return(list(sense=sense, antisense=antisense))
}


samplingMetaGene <- function(features, plusCVG, minusCVG, size=100L, up=10000L,
    down=NULL, nSampling=1000L, samplingRatio=0.1, ...) {
    samplingSize <- round(length(features)*samplingRatio)

    metaList <- mclapply(1:length(features), function(x) {
        metaGene(features=features[x,], plusCVG=plusCVG, minusCVG=minusCVG, 
        size=size, up=up, down=down)
    }, ...)

    allSamples <- mclapply(1:nSampling, function(x) {
        inx <- sample(1:length(features), size=samplingSize, replace=TRUE)
        onesample <- metaList[inx]
        mat <- sapply(onesample, function(x) as.integer(x))
        Rle(apply(mat, 1, sum))
    }, ...)

    M <- sapply(allSamples, function(x) as.integer(x))
    return(Rle(apply(M, 1, median)) / samplingSize)
}


#' Returns a matrix, with rows representing read counts across a specified
#' gene, or other features of interest.
#'
#' Supports parallel processing using mclapply in the 'parallel' package.
#' To change the number of processors, use the argument 'mc.cores'.
#'
#' @param features A GRanges object representing a set of genomic coordinates.
#' @param reads A GRanges object representing a set of mapped reads.
#' @param size The size of the moving window.
#' @param up Distance upstream of each f to align and histogram Default: 1 kb.
#' @param down Distance downstream of each f to align and histogram
#' Default: same as up.
#' @param debug If set to TRUE, provides additional print options.
#' Default: FALSE
#' @param ... Extra argument passed to mclapply
#' @return Returns a vector representing the 'typical' signal across
#' genes of different length.
#' @author Charles G. Danko and Minho Chae
##  Returns a matrix of counts.  Rows represent different streches of DNA.
##  Columns represent positions relative to a certain feature.  Summed together,
##  these should be a meta-gene.
##
##  Arguments:
##  f   -> data.frame of: CHR, START, STRAND.
##  p   -> data.frame of: CHR, START, END, STRAND.
##  size    -> The size of the moving window.
##  up  -> Distance upstream of each f to align and histogram.
##  down    -> Distance downstream of each f to align and histogram (NULL).
##
##  Assumptions: Same as MetaGene
metaGeneMatrix <- function(features, reads, size= 50, up=1000, down=up, 
    debug=FALSE, ...) {

    C <- sort(unique(as.character(seqnames(features))))

    ## Run parallel version.
    mcp <- mclapply(seq_along(C), metaGeneMatrix_foreachChrom, C=C, 
        features=features, reads=reads, size=size, up=up, down=down, 
        debug=debug, ...)
    
    ## Append data from all chromosomes.
    H <- NULL
    for(i in seq_along(C)) {
        # Which KG?  prb?
        indxF   <- which(as.character(seqnames(features)) == C[i])
        indxPrb <- which(as.character(seqnames(reads)) == C[i])

        if((NROW(indxF) >0) & (NROW(indxPrb) >0)) {
            H <- rbind(H, mcp[[i]])
        }
    }

    return(H)
}


metaGeneMatrix_foreachChrom <- function(i, C, features, reads, size, up, down, 
    debug) {
        # Which KG?  prb?
        indxF   <- which(as.character(seqnames(features)) == C[i])
        indxPrb <- which(as.character(seqnames(reads)) == C[i])

        if((NROW(indxF) >0) & (NROW(indxPrb) >0)) {
            # Order -- Make sure, b/c this is one of our main assumptions.  
            # Otherwise violated for DBTSS.
            ord <- order(start(features[indxF,]))
            # Type coersions.
            FeatureStart    <- start(features[indxF,][ord])
            FeatureStr  <- as.character(strand(features[indxF,][ord]))
            PROBEStart  <- start(reads[indxPrb,])
            PROBEEnd    <- end(reads[indxPrb,])
            PROBEStr    <- as.character(strand(reads[indxPrb,]))
            size        <- as.integer(size)
            up          <- as.integer(up)
            down        <- as.integer(down)

            # Set dimensions.
            dim(FeatureStart)   <- c(NROW(FeatureStart), NCOL(FeatureStart))
            dim(FeatureStr)     <- c(NROW(FeatureStr),   NCOL(FeatureStr))
            dim(PROBEStart)     <- c(NROW(PROBEStart),   NCOL(PROBEStart))
            dim(PROBEEnd)       <- c(NROW(PROBEEnd),     NCOL(PROBEEnd))
            dim(PROBEStr)       <- c(NROW(PROBEStr),     NCOL(PROBEStr))

            if(debug) {
                message(C[i],": Counting reads in specified region.")
            }
            Hprime <- .Call("MatrixOfReadsByFeature", FeatureStart, FeatureStr, 
                            PROBEStart, PROBEEnd, PROBEStr, 
                            size, up, down, PACKAGE = "groHMM")
            return(Hprime)
        }
    return(integer(0))
}



#' Returns a histogram of the number of reads in each section of a moving
#' window of #' variable size across genes.
#'
#' Supports parallel processing using mclapply in the 'parallel' package.
#' To change the number of processors, use the argument 'mc.cores'.
#'
#' @param features A GRanges object representing a set of genomic coordinates.
#' @param reads A GRanges object representing a set of mapped reads.
#' @param n_windows The number of windows to break genes into.
#' @param debug If set to TRUE, provides additional print options.
#' Default: FALSE
#' @param ... Extra argument passed to mclapply
#' @return Returns a vector representing the 'typical' signal across genes of
#' different length.
#' @author Charles G. Danko and Minho Chae
#' @examples
#' features <- GRanges("chr7", IRanges(1000, 1100), strand = "+")
#' reads <- GRanges("chr7", IRanges(start = c(1000:1004, 1100),
#'                                  width = rep(1, 6)), strand="+")
#' metaGene_nL(features, reads, n_windows=5)
##  Returns a histogram of the number of reads in each section of a
##  moving window of variable size across genes.
##
##  Arguments:
##  f   -> data.frame of: CHR, START, END, STRAND.
##  p   -> data.frame of: CHR, START, END, STRAND.
##  n_windows   -> The resolution of the MetaGene -- i.e. the number of moving 
##  windows to break it into..
##
##  Assumptions:
##  (1) Gene list should be ordered!  
##  (2) Gene list should be pretty short, as most of the processing and 
##  looping over genes is currently done in R.
#
metaGene_nL <- function(features, reads, n_windows=1000, debug=FALSE, ...) {
    C <- sort(unique(as.character(seqnames(features))))
    H <- rep(0,n_windows)
    for(i in 1:NROW(C)) {
        if(debug) {
            message(C[i])
        }

        # Which KG?  prb?
        indxF   <- which(as.character(seqnames(features)) == C[i])
        indxPrb <- which(as.character(seqnames(reads)) == C[i])

        if((NROW(indxF) >0) & (NROW(indxPrb) >0)) {
            # Order -- Make sure, b/c this is one of our main assumptions.  
            # Otherwise violated for DBTSS.
            ord <- order(start(features[indxF,])) 

            # Type coersions.
            FeatureStart    <- start(features[indxF,][ord])
            FeatureEnd  <- end(features[indxF,][ord])
            FeatureStr  <- as.character(strand(features[indxF,][ord]))
            PROBEStart  <- start(reads[indxPrb,])
            PROBEEnd    <- end(reads[indxPrb,])
            PROBEStr    <- as.character(strand(reads[indxPrb,]))

            # Set dimensions.
            dim(FeatureStart)   <- c(NROW(FeatureStart), NCOL(FeatureStart))
            dim(FeatureStr)     <- c(NROW(FeatureStr),   NCOL(FeatureStr))
            dim(PROBEStart)     <- c(NROW(PROBEStart),   NCOL(PROBEStart))
            dim(PROBEEnd)       <- c(NROW(PROBEEnd),     NCOL(PROBEEnd))
            dim(PROBEStr)       <- c(NROW(PROBEStr),     NCOL(PROBEStr))

            #for(iFeatures in 1:NROW(FeatureStart)) {
            mcpg <- mclapply(c(1:NROW(FeatureStart)), function(iFeatures) {
                ws <- (FeatureEnd[iFeatures]-FeatureStart[iFeatures])/n_windows 
                ## This WILL be an interger.
                if(debug) {
                    message(C[i],": Counting reads in specified region:",
                            FeatureStart[iFeatures],"-",FeatureEnd[iFeatures])
                    message(C[i],": Window size:",
                            FeatureStart[iFeatures],"-",FeatureEnd[iFeatures])
                    message(C[i],": End-Start:",
                            FeatureEnd[iFeatures]-FeatureStart[iFeatures])
                }
                DataByOne <- .Call("WindowAnalysis", PROBEStart, PROBEEnd, 
                        PROBEStr, FeatureStr[iFeatures], as.integer(1), 
                        as.integer(1), FeatureStart[iFeatures], 
                        FeatureEnd[iFeatures], 
                        PACKAGE = "groHMM")

                if(debug) {
                    message("DataByOne size:",NROW(DataByOne))
                }

                ## This seems almost immeidate on my pentium M machine.
                Hprime <- unlist(lapply(1:NROW(H), function(i) {
                    indx <- ceiling(ws*(i-1)+1):ceiling(ws*i)
                    return(sum(DataByOne[indx]))
                }))

                ## Reverse it for "-" strands.
                if(FeatureStr[iFeatures] == "-")
                    Hprime <- rev(Hprime)

                #H <- H + Hprime/ws ## Put in units of: number of reads/base
                return(Hprime/ws)
            }, ...)

            ## Add genes from mclapply together.
            for(iFeatures in 1:NROW(FeatureStart)) { 
                H<- H+mcpg[[i]] 
            }
        }
        if(debug) {
            message(C[i],": Done!")
        }
    }

    return(H)
}


#' Returns the average profile of tiling array probe intensity values or
#' wiggle-like count data centered on a set of genomic positions
#' (specified by 'Peaks').
#'
#' Supports parallel processing using mclapply in the 'parallel' package.
#' To change the number of processors, use the argument 'mc.cores'.
#'
#' @param ProbeData Data.frame representing chromosome, window center,
#' and a value.
#' @param Peaks Data.frame representing chromosome, and window center.
#' @param size Numeric.  The size of the moving window. Default: 50 bp.
#' @param bins The bins of the meta gene -- i.e. the number of moving windows
#' to break it into. Default +/- 1kb from center.
#' @return A vector representing the 'typical' signal centered on the peaks of
#' interest.
#' @author Charles G. Danko and Minho Chae
##  Returns the average profile of tiling array probe intensity values or 
## wiggle-like count data centered on a set of genomic positions.
##
##  Arguments:
##  Peaks       -> data.frame of: CHR, CENTER, STRAND. 
##      (note that STRAND is currenly not supported, and does nothing).
##  ProbeData   -> data.frame of: CHR, CENTER, VALUE
##  bins        -> The bins of the meta gene -- i.e. the number of 
##          moving windows to break it into.
##
##  TODO: 
##  (1) Implement support for a Peaks$starnd 
##  (2) ...
averagePlot <- function(ProbeData, Peaks, size=50, bins= seq(-1000,1000,size)){

    ## For each chromsome.  
    ProbeData$minDist <- rep(999)
    for(chr in unique(Peaks[[1]])) {

      ##   Get the set of data that fits.  
      indxPeaks <- which(Peaks[[1]] == chr)
      
      ## The '$' ensures that chrom is end of line.  Otherwise, grep for chr1 
      ## returns chr10-chr19 as well.
      ## Should work fine, even when chromsome is simply "chrN".
      indxAffxProbes <- grep(paste(chr,"$", sep=""), ProbeData[[1]], perl=TRUE)

      ##   Calculate the minimum distance between the probe and the vector 
      ## over all features ... 
      ProbeData$minDist[indxAffxProbes] <- 
        unlist(lapply(indxAffxProbes, function(x) {
            ## TODO: For strand to switch it, just multiply by strand here.
            return((ProbeData[x,2] - 
                Peaks[indxPeaks,2])[which.min(abs(ProbeData[x,2] - 
                Peaks[indxPeaks,2]))])}))
    }

    ## Make bins.  Take averages and all that...
    means <- unlist(lapply(c(1:NROW(bins)), 
        function(i){mean(ProbeData[(ProbeData$minDist >= 
            (bins[i]-size) & ProbeData$minDist < bins[i]),3])}))
    return(data.frame(windowCenter= bins+(size/2), means))
}

###########################################################################
##
##   Copyright 2013, 2014 Charles Danko and Minho Chae.
##
##   This program is part of the groHMM R package
##
##   groHMM is free software: you can redistribute it and/or modify it
##   under the terms of the GNU General Public License as published by
##   the Free Software Foundation, either version 3 of the License, or
##   (at your option) any later version.
##
##   This program is distributed in the hope that it will be useful, but
##   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
##   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
##   for more details.
##
##   You should have received a copy of the GNU General Public License along
##   with this program.  If not, see <http://www.gnu.org/licenses/>.
##
##########################################################################


#' writeWiggle writes a wiggle track or BigWig file suitable for uploading
#' to the UCSC genome browser.
#'
#' @param reads GenomicRanges object representing the position of reads
#' mapping in the genome.
#' @param file Specifies the filename for output.
#' @param strand Takes values of "+", "-", or "*".  Computes Writes a wiggle
#' on the speicified strand.  "*" denotes collapsing reads on both strands.
#' Default: "*".
#' @param fileType Takes values of "wig" or "BigWig". Default: "wig".
#' @param size Size of the moving window.
#' @param normCounts A normalization factor correcting for library size
#' or other effects.  For example, total mappible read counts might be a
#' reasonable value.  Default: 1 (i.e. no normalization).
#' @param reverse If set to TRUE, multiplies values by -1.
#' Used for reversing GRO-seq data on the negative (-) strand. Default: FALSE
#' @param seqinfo Seqinfo object for reads. Default: NULL.
#' @param track.type.line If set to TRUE, prints a header identifying the
#' file as a wiggle.  Necessary to upload a custom track to the UCSC
#' genome browser.  Default: TRUE
#' @param ...  Extra argument passed to mclapply.
#' @author Minho Chae and Charles G. Danko
#' @examples
#' S0mR1 <- as(readGAlignments(system.file("extdata", "S0mR1.bam",
#' package="groHMM")), "GRanges")
#' ## Not run:
#' # writeWiggle(reads=S0mR1, file="S0mR1_Plus.wig", fileType="wig",
#' # strand="+", reverse=FALSE)
## # library(GenomicRanges)
## # si <- seqinfo(BSgenome.Hsapiens.UCSC.hg19)
## # writeWiggle(reads=S0mR1, file="S0mR1_Plus.wig", fileType="BigWig",
## # strand="+", reverse=FALSE, seqinfo=si)
writeWiggle <- function(reads, file, strand="*", fileType="wig", size=50, 
    normCounts=NULL, reverse=FALSE, seqinfo=NULL, track.type.line=FALSE, ...) {
    W <- windowAnalysis(reads = reads, strand = strand, windowSize = size, ...)
    if (is.null(normCounts)) {
        normCounts <- 1
    }
    if (reverse) {
        normCounts <- (-1L)*normCounts
    }

    append <- FALSE
    trackWritten <- FALSE
    writeBlock <- function(chr) {
        reads_s <- GRanges(chr, 
            IRanges(seq(1, by=size, length.out=length(W[[chr]])),
            seq(size, by=size, length.out=length(W[[chr]]))),
            strand=strand, score=normCounts*as.integer(W[[chr]]))
        if (track.type.line & !trackWritten) {
            cat("track type=wiggle_0\n", file=file)
            append <<- TRUE
        }
        export(reads_s, file, format="wig", dataFormat="fixedStep", 
            append=append)
        append <<- TRUE
    }
    lapply(names(W), writeBlock)

    if (fileType=="BigWig") {
        wigToBigWig(file, seqinfo)
    }
}

###########################################################################
##
##   Copyright 2013, 2014 Charles Danko and Minho Chae.
##
##   This program is part of the groHMM R package
##
##   groHMM is free software: you can redistribute it and/or modify it 
##   under the terms of the GNU General Public License as published by 
##   the Free Software Foundation, either version 3 of the License, or  
##   (at your option) any later version.
##
##   This program is distributed in the hope that it will be useful, but 
##   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
##   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
##   for more details.
##
##   You should have received a copy of the GNU General Public License along 
##   with this program.  If not, see <http://www.gnu.org/licenses/>.
##
##########################################################################



#' Function identifies expressed features using the methods introduced in
#' Core, Waterfall, Lis; Science, Dec. 2008.
#'
#' Supports parallel processing using mclapply in the 'parallel' package.
#' To change the number of processors use the argument 'mc.cores'.
#'
#' @param features A GRanges object representing a set of genomic coordinates.
#' The meta-plot will be centered on the start position.
#' There can be optional "ID" column for gene ids.
#' @param reads A GRanges object representing a set of mapped reads.
#' @param Lambda Measurement of assay noise.  Default: 0.04 reads/ kb in a
#' library of 10,751,533 mapped reads. (background computed in Core,
#' Waterfall, Lis. (2008) Science.).
#' @param UnMap List object representing the position of un-mappable reads.
#' Default: not used.
#' @param debug If set to true, returns the number of positions.
#' Default: FALSE.
#' @param ... Extra argument passed to mclapply
#' @return Returns a data.frame representing the expression p.values for
#' features of interest.
#' @author Charles G. Danko
#' @examples
#' features <- GRanges("chr7", IRanges(2394474, 2420377), strand = "+",
#'                     ID = "gene1")
#' reads <-
#'     system.file("extdata", "S0mR1.bam", package = "groHMM") |>
#'     readGAlignments() |>
#'     as("GRanges")
#' expressedGenes(features, reads)
##  This identifes genes that are expressed in a given cell, based on short 
##  read data.
##      f  == genes/annotations; columns represent: Chr, Start, End, Strand, ID.
##      p  == short reads; columns represent: Chr, Start, End, Strand, ID.
##      UnMAQ == unmappable regions in the genome of interest.
##
##  Function defines expression as in Core, Waterfall, Lis; Science, Dec. 2008.
expressedGenes <- function(features, reads, Lambda= NULL, UnMap=NULL, 
    debug=FALSE, ...) {
    # Order -- Make sure, b/c this is one of our main assumptions.  Otherwise 
    # violated for DBTSS.
    reads <- reads[order(as.character(seqnames(reads)), start(reads)),] 
    C <- sort(unique(as.character(seqnames(features))))
    if(is.null(Lambda)) Lambda <- 0.04*NROW(reads)/10751533/1000 
    #NROW(reads)/genomeSize
    
    ## Run parallel version.
    mcp <- mclapply(seq_along(C), expressedGenes_foreachChrom, C=C, 
            features=features, reads=reads, Lambda=Lambda, UnMap=UnMap, 
            debug=debug, ...)

    ## Unlist... 
    ANSgeneid <- rep("char", NROW(features))
    ANSpvalue <- rep(0,NROW(features))
    ANScounts <- rep(0,NROW(features))
    ANSunmapp <- rep(0,NROW(features))
    ANSgsize  <- rep(0,NROW(features))
    for(i in seq_along(C)) {
        indxF   <- which(as.character(seqnames(features)) == C[i])
        indxPrb   <- which(as.character(seqnames(reads)) == C[i])
        if((NROW(indxF) >0) & (NROW(indxPrb) >0)) {
             ANSgeneid[indxF][mcp[[i]][["ord"]]] <- mcp[[i]][["ANSgeneid"]]
             ANSpvalue[indxF][mcp[[i]][["ord"]]] <- mcp[[i]][["ANSpvalue"]]
             ANScounts[indxF][mcp[[i]][["ord"]]] <- mcp[[i]][["ANScounts"]]
             ANSunmapp[indxF][mcp[[i]][["ord"]]] <- mcp[[i]][["ANSunmapp"]]
             ANSgsize[indxF][mcp[[i]][["ord"]]] <- mcp[[i]][["ANSgsize"]]
        }
    }

    return(data.frame(ID= ANSgeneid, pval= ANSpvalue, readCounts= ANScounts, 
                    nonMappablePositions= ANSunmapp, size= ANSgsize))
}
 
expressedGenes_foreachChrom <- function(i, C, features, reads, Lambda, UnMap, 
    debug) {
        if(debug) {
            message("Doing chromosome ", C[i])
        }
    
        # Which KG?  prb?
        indxF   <- which(as.character(seqnames(features)) == C[i])
        indxPrb <- which(as.character(seqnames(reads)) == C[i])

        if((NROW(indxF) >0) & (NROW(indxPrb) >0)) {
            # Order -- Make sure, b/c this is one of our main assumptions.  
            # Otherwise violated for DBTSS.
            Ford <- order(start(features[indxF,])) 

            # Type coersions.
            FeatureStart    <- start(features[indxF,][Ford])
            FeatureEnd  <- end(features[indxF,][Ford])
            FeatureStr  <- as.character(strand(features[indxF,][Ford]))
            PROBEStart  <- start(reads[indxPrb,])
            PROBEEnd    <- end(reads[indxPrb,])
            PROBEStr    <- as.character(strand(reads[indxPrb,]))

            # Set dimensions.
            dim(FeatureStart)   <- c(NROW(FeatureStart), NCOL(FeatureStart))
            dim(FeatureEnd)     <- c(NROW(FeatureEnd),   NCOL(FeatureEnd))
            dim(FeatureStr)     <- c(NROW(FeatureStr),   NCOL(FeatureStr))
            dim(PROBEStart)     <- c(NROW(PROBEStart),   NCOL(PROBEStart))
            dim(PROBEEnd)       <- c(NROW(PROBEEnd),     NCOL(PROBEEnd))
            dim(PROBEStr)       <- c(NROW(PROBEStr),     NCOL(PROBEStr))

            
            NUMReads <- .Call("CountReadsInFeatures", FeatureStart, 
                            FeatureEnd, FeatureStr, PROBEStart, PROBEEnd, 
                            PROBEStr, PACKAGE = "groHMM")


            ## Calculate UN-MAQable regions...
            if(!is.null(UnMap)) {

                ## Count start index.
                chr_indx <- which(UnMap[[1]][[1]] == C[i])
                CHRSIZE <- as.integer(UnMap[[1]][[2]][chr_indx])
                CHRSTART <- as.integer(0)
                if(chr_indx > 1) {  ## Running on 1:0 gives c(1, 0)
                    CHRSTART <- as.integer( 
                        sum(UnMap[[1]][[2]][
                            c(1:(chr_indx-1))
                        ]) +1)
                }

                if(debug) {
                    message(C[i],": Counting unMAQable regions.")
                    message("CHRSIZE:", CHRSIZE, "CHRSTART:", CHRSTART)
                }

                ## Count unMAQable regions, and size of everything ... 
                nonmappable <- .Call("CountUnMAQableReads", FeatureStart, 
                    FeatureEnd, UnMap[[2]], CHRSTART, CHRSIZE, 
                    PACKAGE = "groHMM")

                ## Adjust size of gene body.
                MappablePositions <- (FeatureEnd - FeatureStart) - 
                                nonmappable + 1

                if(debug) {
                    print(summary(nonmappable))
                    print(summary(MappablePositions))
                }
            }
            else {
                nonmappable <- 1
            }
            
            MappablePositions <- (FeatureEnd - FeatureStart) - nonmappable + 1

            ## Calculate poisson prob. of each.
            if ("ID" %in% colnames(mcols(features))) {  # there is "ID" column
                ANSgeneid_c <- elementMetadata(features[indxF,])$ID
            } else {
                ANSgeneid_c <- rep(NA, NROW(indxF)) 
            }
             ANSpvalue_c <- ppois(NUMReads, (Lambda*MappablePositions), 
                                    lower.tail=FALSE)
             ANScounts_c <- NUMReads
             ANSunmapp_c <- nonmappable
             ANSgsize_c <- (FeatureEnd-FeatureStart) #[indxF][Ford]
            
            return(list(ANSgeneid= ANSgeneid_c, ANSpvalue= ANSpvalue_c, 
                    ANScounts= ANScounts_c, ANSunmapp=  ANSunmapp_c,
                    ANSgsize= ANSgsize_c, ord= Ford))
        }
    return(integer(0))
}

###########################################################################
##
##   Copyright 2013, 2014 Charles Danko and Minho Chae.
##
##   This program is part of the groHMM R package
##
##   groHMM is free software: you can redistribute it and/or modify it
##   under the terms of the GNU General Public License as published by
##   the Free Software Foundation, either version 3 of the License, or
##   (at your option) any later version.
##
##   This program is distributed in the hope that it will be useful, but
##   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
##   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
##   for more details.
##
##   You should have received a copy of the GNU General Public License along
##   with this program.  If not, see <http://www.gnu.org/licenses/>.
##
##########################################################################

#' getTxDensity Calculates transcript density.
#'
#' Calculates transcript density for transcripts which overlapps with
#' annotations.
#' For 'run genes together' or 'broken up a single annotation' errors,
#' best overlapped transcripts or annotations are used.
#'
#' Supports parallel processing using mclapply in the 'parallel' package.
#' To change the number of processors
#' set the option 'mc.cores'.
#'
#' @param tx GRanges of transcripts.
#' @param annox GRanges of non-overlapping annotatoins.
#' @param plot Logical.  If TRUE, plot transcript density.  Default: TRUE
#' @param scale Numeric. Scaled size of a gene for transcript density
#' calculation.
#' Default: 1000L
#' @param nSampling Numeric. Number of subsampling.  Default: 0L
#' @param samplingRatio Numeric. Ratio of sampling for annotations.
#' Default: 0.1
#' @param ... Extra argument passed to mclapply.
#' @return Returns a list of FTD, TTD, PostTTS, and TUA.
#' @author Minho Chae
#' @examples
#' tx <- GRanges("chr7", IRanges(start=seq(1000,4000, by=1000),
#' width=seq(1000, 1300, by=100)), strand=rep("+", 4))
#' annox <- GRanges("chr7", IRanges(start=seq(1100,4100, by=1000),
#' width=seq(900, 1200, by=100)), strand=rep("+", 4))
#' ## Not run:
#' # density <- getTxDensity(tx, annox)
getTxDensity <- function(tx, annox, plot=TRUE, scale=1000L, nSampling=0L, 
    samplingRatio=0.1, ...) {
    ol <- findOverlaps(tx, annox)

    # Count tx
    runGenes <- unique(queryHits(ol[duplicated(queryHits(ol)),]))       
    # Count annox
    brokenUp <- unique(subjectHits(ol[duplicated(subjectHits(ol)),]))   
    cat("Merged annotations: ", length(runGenes), "\n")
    cat("Dissociated a single annotation: ", length(brokenUp), "\n")
    cat("Overlaps between transcript and annotation:", "\n")
    cat("Total = ", length(ol))

    # For each annox, find the best matching tx, runGenes case...
    intx_rg <- pintersect(tx[queryHits(ol),], annox[subjectHits(ol),])
    intx_rg_df <- data.frame(tx=queryHits(ol), annox=subjectHits(ol),   
            oRatio=width(intx_rg)/width(tx[queryHits(ol),]))

    #Tx matches multiple times on a same annox
    remove_rg <- unique(unlist(lapply(runGenes, function(x) {
                    inx <- which(queryHits(ol) == x)
                    m <- which.max(intx_rg_df$oRatio[inx])
                    inx[-m]
                })))

    if (length(remove_rg) > 0)  
        ol <- ol[-remove_rg,]

    # For each transcript, find the best matching annox, brokenUp case..., 
    brokenUp <- unique(subjectHits(ol[duplicated(subjectHits(ol)),]))

    intx_bu  <- pintersect(tx[queryHits(ol),], annox[subjectHits(ol),])
    intx_bu_df <- data.frame(tx=queryHits(ol), annox=subjectHits(ol),   
            oRatio=width(intx_bu)/width(annox[subjectHits(ol),]))


    # Annox matches multiple times on a same transcript
    remove_bu <- unique(unlist(lapply(brokenUp, function(x) {
                    inx <- which(subjectHits(ol) == x)
                    m <- which.max(intx_bu_df$oRatio[inx])
                    inx[-m]
                })))
    if (length(remove_bu) > 0)  
        ol <- ol[-remove_bu,]

    cat(" Used for density = ", length(ol), "\n")

    olTx <- tx[queryHits(ol),]
    # Now get the coverage of selected transcripts
    olStrand <- as.character(strand(tx[queryHits(ol),]))
    olChrom <- seqnames(tx[queryHits(ol),])

    # Get the extended region for annox
    up <- 1L
    down <- 2L
    message("Calculate overlapping ... ", appendLF=FALSE)
    promo  <- unlist(GRangesList(mclapply(subjectHits(ol), function(x) {
                        w <- width(annox[x,])
                        promoters(annox[x,], upstream=round(w*up), 
                            downstream=round(w*down))
                    }, ... )))

    pintx <- pintersect(promo, olTx)

    # Get theoverlapped coverage
    olcvg <- mclapply(1:length(ol), function(x) {
                    t <- olTx[x,]
                    p <- promo[x,]
                    i <- pintx[x,]
                    # Position is relative to the minimum start 
                    minStart <- min(start(t), start(p))
                    t <- shift(t, -minStart+1)
                    p <- shift(p, -minStart+1)
                    i <- shift(i, -minStart+1)
                    r <- reduce(c(t, p, ignore.mcols=TRUE))
                    rTF <- logical(length=width(r))
                    rTF[start(i):end(i)] <- TRUE
                    if (olStrand[x] == "+")
                        Rle(rTF[start(p):end(p)])
                    else
                        rev(Rle(rTF[start(p):end(p)]))
            }, ...)
    message("OK")
                
    message("Scale overlapping ... ", appendLF=FALSE)
    # Get the scaled coverage 
    cvgWidth <- round(up*scale) + round(down*scale)
    sccvg <- mclapply(olcvg, function(x) {
                        getLIValues(x, cvgWidth)
                    }, ... )
    message("OK")

    M <- sapply(sccvg, function(x) as.integer(x))
    sSize <- round(length(ol)*samplingRatio)
    if (nSampling > 0) {
        message("Sampling ... ", appendLF=FALSE)
        allSamples <- mclapply(1:nSampling, function(x) {
            inx <- sample(1:length(ol), size=sSize, replace=TRUE)
            onesample <- M[,inx]
            Rle(apply(onesample, 1, sum))
        }, ...)
        mat <- sapply(allSamples, function(x) as.integer(x))
        profile <- apply(mat, 1, mean)/sSize
        message("OK")
    } else {
        profile <- apply(M, 1, sum)/length(ol)
    }
    if (plot) {
        plot(-(up*scale):(down*scale-1), profile, ylim=c(0, 1), type="l", 
            xlab="Relative to TSS", ylab="Density")
        abline(v=0, col="blue", lty=2)
        abline(v=up*scale, col="blue", lty=2)
    }

    trap.rule <- function(x,f) {sum(diff(x)*(f[-1]+f[-length(f)]))/2}
    # Thanks to: http://tolstoy.newcastle.edu.au/R/help/05/08/9625.html

    FivePrimeFP <- trap.rule(1:scale, profile[1:scale])/scale
    TP <- trap.rule(1:scale, profile[(scale+1):(scale*2)])/scale
    PostTTS <- trap.rule(1:scale, profile[(scale*2+1):(scale*3)])/scale
    TUA <- (TP + (TP- FivePrimeFP))/(1 + TP)
    return(list(FivePrimeFP=FivePrimeFP, TP=TP, PostTTS=PostTTS, TUA=TUA))

}

getWP <- function (lv, lw) {
    return( ((lv-1)/(lw-1)) * (0:(lw-1)) + 1)
}

getLIValue <- function (x0, y0, x1, y1, x) {
    alpha = (x - x0) / (x1 - x0)
    return(y0 + alpha * (y1 - y0))
}

# Get linear interpolation data
getLIValues <- function (vals, n) {
    vals <- as.integer(vals)
    wp <- getWP(length(vals), n)
    #print(wp)
    result <- seq(n)
    if (length(vals) == 1) {
        result[1:length(result)] <- vals
        return(result)
    }

    result[1] <- vals[1]
    result[length(result)] <- vals[length(vals)]

    if (n > 2) {
        for(i in 2:(n-1)) {
            x <- wp[i]
            x0 <- floor(x)
            result[i] <- getLIValue(x0, vals[x0], x0+1, vals[x0+1], x)
        }
    }
    return(Rle(round(result)))
}



#' evaluateHMM Evaluates HMM calling.
#'
#' Evaluates HMM calling of transripts compared to known annotations.
#'
#' @param tx GRanges of transcripts predicted by HMM.
#' @param annox GRanges of non-overlapping annotatoins.
#' @return a list of error information; merged annotations, dissociated annotation,
#' total, and rate.
#' @author Minho Chae
#' @examples
#' tx <- GRanges("chr7", IRanges(start=seq(100, 1000, by=200),
#' width=seq(100, 1000, by=100)), strand="+")
#' annox <- GRanges("chr7", IRanges(start=seq(110, 1100, by=150),
#' width=seq(100, 1000, by=150)), strand="+")
#' error <- evaluateHMMInAnnotations(tx, annox)
evaluateHMMInAnnotations <- function (tx, annox) {
    o <- findOverlaps(tx, annox)
    # count tx
    merged <- length(unique(queryHits(o[duplicated(queryHits(o)),])))  
    # count annox
    dissociated <- length(unique(subjectHits(o[duplicated(subjectHits(o)),]))) 

    eval <- data.frame(merged=merged, dissociated=dissociated,
            total=(merged+ dissociated), errorRate=(merged + dissociated)/
                (length(tx) + length(annox)),
            txSize =length(tx))

    intx <- pintersect(tx[queryHits(o),], annox[subjectHits(o),])
    overlap <- data.frame(txRatio=width(intx)/width(tx[queryHits(o),]),
                annoxRatio= width(intx)/width(annox[subjectHits(o),]),
                overBases=width(intx), 
                similarity=width(intx)/pmax(width(tx[queryHits(o),]), 
               width(annox[subjectHits(o),])))

    return(list(eval=eval, overlap=overlap))
}

###########################################################################
##
##   Copyright 2013, 2014 Charles Danko and Minho Chae.
##
##   This program is part of the groHMM R package
##
##   groHMM is free software: you can redistribute it and/or modify it
##   under the terms of the GNU General Public License as published by
##   the Free Software Foundation, either version 3 of the License, or
##   (at your option) any later version.
##
##   This program is distributed in the hope that it will be useful, but
##   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
##   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
##   for more details.
##
##   You should have received a copy of the GNU General Public License along
##   with this program.  If not, see <http://www.gnu.org/licenses/>.
##
##########################################################################

##
##  TODO: Re-factor to one function, allowing the model version to be specified 
##  as an argument (Charles).
##
##
##

#' Given GRO-seq data, identifies the location of the polymerase 'wave' in
#' up- or down- regulated genes.
#'
#' The model is a three state hidden Markov model (HMM).  States represent:
#' (1) the 5' end of genes upstream of the transcription start site,
#' (2) upregulated sequence, and (3) the 3' end of the gene through the
#' polyadenylation site.
#'
#' The model computes differences in read counts between the two conditions.
#' Differences are assumed fit a functional form which can be specified by
#' the user (using the emissionDistAssumption argument).
#' Currently supported functional forms include a normal distribution
#' (good for GRO-seq data prepared using the circular ligation protocol),
#' a gamma distribution (good for 'spikey' ligation based GRO-seq data),
#' and a long-tailed normal+exponential distribution was implemented, but
#' never deployed.
#'
#' Initial parameter estimates are based on initial assumptions of
#' transcription rates taken from the literature.  Subsequently all parameters
#' are fit using Baum-Welch expetation maximization.
#'
#' Reference: Danko CG, Hah N, Luo X, Martins AL, Core L, Lis JT, Siepel A,
#' Kraus WL. Signaling Pathways Differentially Affect RNA Polymerase II
#' Initiation, Pausing, and Elongation Rate in Cells. Mol Cell.
#' 2013 Mar 19. doi:pii: S1097-2765(13)00171-8. 10.1016/j.molcel.2013.02.015.
#'
#' Arguments:
#' @param reads1 Mapped reads in time point 1.
#' @param reads2 Mapped reads in time point 2.
#' @param genes A set of genes in which to search for the wave.
#' @param approxDist The approximate position of the wave.  Suggest using 2000
#'   [bp/ min] * time [min], for mammalian data.
#' @param size The size of the moving window. Suggest using 50 for direct
#'   ligation data, and 200 for circular ligation data.  Default: 50.
#' @param upstreamDist The amount of upstream sequence to include Default: 10
#'   kb.
#' @param TSmooth Optimonally, outlying windows are set a maximum value over
#'   the inter-quantile interval, specified by TSmooth.  Reasonable value: 20;
#'   Default: NA (for no smoothing).  Users are encouraged to use this
#'   parameter ONLY in combination with the normal distribution assumptions.
#' @param NonMap Optionally, un-mappable positions are trated as missing data.
#'   NonMap passes in the list() structure for un-mappable regions.
#' @param prefix Optionally, writes out png images of each gene examined for a
#'   wave.  'Prefix' denotes the file prefix for image names written to disk.
#'   Users are encouraged to create a new directory and write in a full path.
#' @param emissionDistAssumption Takes values "norm", "normExp", and "gamma".
#'   Specifies the functional form of the 'emission' distribution for states I
#'   and II (i.e. 5' of the gene, and inside of the wave).  In our experience,
#'   "gamma" works best for highly-variable 'spikey' data, and "norm" works for
#'   smooth data.  As a general rule of thumb, "gamma" is used for libraries
#'   made using the direct ligation method, and "norm" for circular ligation
#'   data.  Default: "gamma".
#' @param finterWindowSize Method returns 'quality' information for each gene
#'   to which a wave was fit.  Included in these metrics are several that
#'   define a moving window.  The moving window size is specified by
#'   filterWindowSize.  Default: 10 kb.
#' @param limitPCRDups If true, counts only 1 read at each position with >= 1
#'   read.  NOT recommended to set this to TRUE.  Defulat: FALSE.
#' @param returnVal Takes value "simple" (default) or "alldata". "simple"
#'   returns a data.frame with Pol II wave end positions.  "alldata" returns
#'   all of the availiable data from each gene, including the full posterior
#'   distribution of the model after EM.
#' @param debug If TRUE, prints error messages.
#' @return Returns either a data.frame with Pol II wave end positions, or a
#'   List() structure with additional data, as specified by returnVal.
#' @author Charles G. Danko
#' @examples
#' genes <- GRanges("chr7", IRanges(2394474,2420377), strand="+",
#' SYMBOL="CYP2W1", ID="54905")
#' reads1 <- as(readGAlignments(system.file("extdata", "S0mR1.bam",
#'                             package="groHMM")), "GRanges")
#' reads2 <- as(readGAlignments(system.file("extdata", "S40mR1.bam",
#'                             package="groHMM")), "GRanges")
#' approxDist <- 2000*10
#' # Not run:
#' # pw <- polymeraseWave(reads1, reads2, genes, approxDist)
## Given GRO-seq data, identifies the location of the polymerase wave in up- or
## down-regulated genes.  This version is based on a full Baum-Welch EM
## implementation.
##
## This is a three state HMM -- initial state representing the intergenic
## region 5' of a gene, the second representing the initially upregulated
## region, and the third representing the remaining sequence of a gene.
##
## We assume that upstream region is intergenic, and thus its emmission
## distriubtion is assumed to be a constant, set based on a representative
## intergenic region.  This is accomidated in my [1,*) framework by keeping the
## variance constant, and scaling the mean for each gene.
##
## Test with GREB1:     chr2:11,591,693-11,700,363
## GREB1 <- data.frame(chr="chr2", start=11591693, end=11700363, str="+")
polymeraseWave <- function(reads1, reads2, genes, approxDist, size=50, 
    upstreamDist= 10000, TSmooth=NA, NonMap=NULL, prefix=NULL, 
    emissionDistAssumption= "gamma", finterWindowSize=10000, 
    limitPCRDups=FALSE, returnVal="simple", debug=TRUE) {
    if(debug) {
        message("Analyzing windows")
    }   

    genes <- as.data.frame(genes)
    genes <- genes[,c("seqnames", "start", "end", "strand", "SYMBOL", "ID")]

    Fp1 <- windowAnalysis(reads=reads1, strand="+", windowSize=size, 
        limitPCRDups=limitPCRDups)
    Fp2 <- windowAnalysis(reads=reads2, strand="+", windowSize=size, 
        limitPCRDups=limitPCRDups)
    Fm1 <- windowAnalysis(reads=reads1, strand="-", windowSize=size, 
        limitPCRDups=limitPCRDups)
    Fm2 <- windowAnalysis(reads=reads2, strand="-", windowSize=size, 
        limitPCRDups=limitPCRDups)
    sizeP1 <- NROW(reads1)
    sizeP2 <- NROW(reads2)
    expCounts <- mean(NROW(reads1),NROW(reads2))

    ANS <- rep(-1, NROW(genes))
    ENDwave <- rep(-1,NROW(genes))
    STRTwave <- rep(-1,NROW(genes))
    KLdivFinal <- rep(-1,NROW(genes))
    KLdivPar <- rep(-1,NROW(genes))
    minWindLTMed <- rep(FALSE,NROW(genes))
    minMeanWindLTMed <- rep(FALSE,NROW(genes))
    nstates<-as.integer(3)    # number of states in HMM.
    unmap <- NA
    
    ## Possible return value.
    dataList <- list()

## Run the model separately on each gene.
    for(i in seq_along(NROW(genes))) {
        geneData <- list()

        if(debug) {
            message("Starting HMM: ", genes[i,5])
        }

    ############################################################################
    #### Define the gene in terms of the windowed size.
        ## Pull the data for the gene.
        if(genes[i,4] == "+") {
            start <- floor((genes[i,2]-upstreamDist)/size)
            end   <- ceiling(genes[i,3]/size)
            emis1  <- 
                (as.numeric(Fp1[[ as.character(genes[i,1]) ]]))[c(start:end)]
                #/sizeP1*expCounts
            emis2  <- 
                (as.numeric(Fp2[[ as.character(genes[i,1]) ]]))[c(start:end)]
                #/sizeP2*expCounts
        }
        else {
            start <- floor(genes[i,2]/size)
            end   <- ceiling((genes[i,3]+upstreamDist)/size)
            emis1  <- 
                rev((as.integer(Fm1[[ as.character(genes[i,1]) ]]))
                    [c(start:end)])#/sizeP1*expCounts
            emis2  <- 
                rev((as.integer(Fm2[[ as.character(genes[i,1]) ]]))
                    [c(start:end)])#/sizeP2*expCounts
        }
    
        ## Scale to a minimum of 1 read at each position (for fitting Gamma). 
        gene  <- as.numeric(emis1 - emis2)
        if(emissionDistAssumption == "gamma") { 
            ## Leave centered on 0 for the norm_exp/norm emission functions
          gene  <- gene +(-1)*(min(gene))+1 
          ## Must translate points if gamma distributed (gamma undefined <0).
        }
        
        if(is.double(TSmooth)) { ## Interperts it as a fold over the inter 
                                 ## quantile interval to filter.
            message("TSmooth is.integer:", TSmooth)
            medGene <- median(gene)
            iqrGene <- IQR(gene)
            gene[(medGene-gene)>(TSmooth*(iqrGene+1))] <- 
                medGene-(TSmooth*(iqrGene+1))
            gene[(gene-medGene)>(TSmooth*(iqrGene+1))] <- 
                medGene+(TSmooth*(iqrGene+1))
        } else if(!is.na(TSmooth)) {
           gene  <- smooth(gene, kind=TSmooth)
        }

        #write.table(gene, "TMP.gene.Rflat")

#       uTrans<- as.integer(ceiling((upstreamDist)/size))
##      Make the initial guess +5kb --> approxDist.
        uTrans<- as.integer(ceiling((upstreamDist-5000)/size))
        iTrans<- as.integer(ceiling((upstreamDist+approxDist)/size))

        ## Run Baum-Welch
        if(debug) message("initial guess:", uTrans, iTrans, NROW(gene))
        counter <- 0

    ###########################################################################
    ## Calculate a moving average and moving max.

        # For each point.  Left of point is defined as P, right is defined as Q.
        # From min(gene):max(gene).
        # Calculate histogram of points
#      if(!is.null(prefix)) { ## Slow, but required for MinOfMax/Avg filter(s).
        MovMeanSpd <- finterWindowSize#10000

        KLdiv <- rep(0,NROW(gene))
        KS    <- rep(0,NROW(gene))
        Means <- rep(0,NROW(gene))

        MovMean  <- rep(0,NROW(gene))
        MovMax   <- rep(0,NROW(gene))
        dMovMean <- rep(0,NROW(gene))

        for(k in c(2:(NROW(gene)-2))) {
            left  <- gene[c(1:k)]
            right <- gene[c((k+1):NROW(gene))]

            LeftHist  <- hist(left, breaks=c((min(gene, na.rm=TRUE)-1)
                :(max(gene, na.rm=TRUE)+1)), plot=FALSE)
            RightHist <- hist(right, breaks=c((min(gene, na.rm=TRUE)-1)
                :(max(gene, na.rm=TRUE)+1)), plot=FALSE)

            minD <- 0.000001
            KLdiv[k]  <- sum((LeftHist$density*log(
                        (LeftHist$density+minD)/(RightHist$density+minD))))
            KS[k]     <- ks.test(left,right)$statistic[[1]]
            Means[k]  <- mean(left, na.rm=TRUE)-mean(right, na.rm=TRUE)

            MovMean[k] <- mean(gene[max((k-(MovMeanSpd/size)),1)
                :min((k+(MovMeanSpd/size)),NROW(gene))], na.rm=TRUE)
            MovMax[k]  <-  max(gene[max((k-(MovMeanSpd/size)),1)
                :min((k+(MovMeanSpd/size)),NROW(gene))], na.rm=TRUE)
            dMovMean[k]   <- MovMean[k-1] - MovMean[k]
        }
#        }

    ############################################################################
    #### Set up initial paremeter estimates.

        ## Fit transition and initial probabilities.
        tProb  <- as.list(data.frame(
            log(c((1-(1/uTrans)),(1/uTrans),0)),
            log(c(0,(1-(1/(iTrans-uTrans))),(1/(iTrans-uTrans)))), 
            log(c(0, 0, 1))))  # Trans. prob.
        iProb  <- as.double(log(c(1, 0, 0))) # iProb.

        ## Fit initial distribution paremeters for emission probabilities.
        parInt  <- Rnorm(gene[c(1:uTrans)])
        if(is.na(parInt$var) | parInt$var == 0) parInt$var = 0.00001 
        ## Check that the varience of the intergenic state is NOT 0.

        if(emissionDistAssumption == "norm") {
            ePrDist <- c("norm", "norm", "norm") 
            #      ePrDist <- c("norm", "normexp", "normexp")
            parPsi  <- Rnorm(gene[c((uTrans+1):iTrans)])
            #Rnorm.exp(gene[c((uTrans+1):iTrans)], tol=1e-4) #
            parBas  <- Rnorm(gene[c((iTrans+1):NROW(gene))])
            #Rnorm.exp(gene[c((iTrans+1):NROW(gene))], tol=1e-4) #
            ePrVars <- data.frame(c(parInt$mean, 
                sqrt(parInt$var), -1, -1), 
                c(parPsi$mean, sqrt(parPsi$var), -1, -1), 
                c(parBas$mean, sqrt(parBas$var), -1, -1))
        }
        else if(emissionDistAssumption == "normExp") {
            ePrDist <- c("norm", "normexp", "normexp")
            parPsi  <- Rnorm.exp(gene[c((uTrans+1):iTrans)], tol=1e-4) #
            parBas  <- Rnorm.exp(gene[c((iTrans+1):NROW(gene))], tol=1e-4) #
            ePrVars <- data.frame(c(parInt$mean, sqrt(parInt$var), -1, -1),
                    parPsi$parameters, parBas$parameters)
        }
        else if(emissionDistAssumption == "gamma") {
            ePrDist <- c("norm", "gamma", "gamma")
            parPsi  <- RgammaMLE(gene[c((uTrans+1):iTrans)])
            parBas  <- RgammaMLE(gene[c((iTrans+1):NROW(gene))])
            ePrVars <- data.frame(c(parInt$mean, sqrt(parInt$var), -1), 
                c(parPsi$shape, parPsi$scale, -1), 
                c(parBas$shape, parBas$scale, -1))
        }
        else {
          message("emissionDistAssumption should be set to: 'norm', 
            'normExp', or 'gamma'.")
          stopifnot(FALSE) ## Stop here.
        }

#           print(data.frame(c(iMean, parInt$var, -1),c(shape, scale, -1),
#           c(meanBas, stdeBas, -1)))
    ###########################################################################
    ## Finally, choose non-mappable windows and treat them as missing values.
        if(!is.null(NonMap)) {
            windowStarts <- seq(start, end, size) 
            ## Start and end are like chromStart and chromEnd -- 
            ## strand invariant.
            windowEnds <- windowStarts+size
            windowsToSurvey <- data.frame(chrom= rep(as.character(genes[i,1]),
                NROW(windowStarts)), chromStart= as.integer(windowStarts), 
                chromEnd= as.integer(windowEnds), strand= 
                    rep("+",NROW(windowStarts)))
            print(head(windowsToSurvey))
            print(tail(windowsToSurvey))
            unmap <- countMappableReadsInInterval(windowsToSurvey, NonMap)
            if(genes[i,4] == "+") {
                unmap <- rev(unmap)
            }
            gene[unmap/size < 0.10] <- NaN 
            ## Missing data if more than 90% of the windows are unmappable.
    }

    ############################################################################
    ## Now run the HMM.
        if(debug) {
            print(ePrVars)
            print(tProb)
        }

        g <- list()
        g[[1]] <- gene
        ans <- list()
        ans <- tryCatch(.Call("RBaumWelchEM", nstates, g, as.integer(1), 
                ePrDist, ePrVars, tProb, iProb, 0.01, c(TRUE,TRUE,TRUE), 
                c(TRUE, TRUE, TRUE), as.integer(10), TRUE, PACKAGE="groHMM"), 
                error=function(e) e)
                                        ##  Update emis...
        if(NROW(ans) < 3) { 
            ## An error will have a length of 2 (is this guaranteed?!).  
            print("ERROR CAUGHT ON THE C SIDE")
            print(ans) 
            ### Will be a previous iteration of ans...?! Generated a new 'ans'
            ans[[1]] <- NA
            ans[[2]] <- NA
            ans[[3]] <- c(rep(0, NROW(gene)-2), 1, 2)
            ans[[4]] <- NA
            ans[[5]] <- NA
        }
        
        ansVitervi <- ans[[3]][[1]]
        DTs <- max(which(ansVitervi==0))
        DTe <- max(which(ansVitervi==1))

        ### Calculate quality filters... Wrap up.
        KLdivHMM <- 0
        if(debug) print(paste("EM Converged to: ",DTs,DTe,NROW(gene)))
        if((DTs >= 1) & (DTe > 1) & (DTs < DTe) & 
            (DTe < NROW(gene)) & (DTs < NROW(gene))) { 
            ## iff convergest to something useful.
            ## Refit and calculate KL-divergence at that point.
            pINew <- Rnorm(gene[c(1:DTs)])
            
            if(emissionDistAssumption == "norm") {
                pPNew <- Rnorm(gene[c((uTrans+1):iTrans)]) 
                # Rnorm.exp(gene[c((uTrans+1):iTrans)], tol=1e-4) 
                #Rnorm(gene[c((uTrans+1):iTrans)])
                pBNew <- Rnorm(gene[c((DTe+1):NROW(gene))])     
                #Rnorm.exp(gene[c((DTe+1):NROW(gene))], tol=1e-4) #

                ## Estimate KL-divergence.
                PSI <- dnorm(c(min(gene):max(gene)), pBNew$mean, 
                    sqrt(pBNew$var))
                BAS <- dnorm(c(min(gene):max(gene)), pBNew$mean, 
                    sqrt(pBNew$var))
            }
            else if(emissionDistAssumption == "normExp") {
                pPNew <- Rnorm.exp(gene[c((uTrans+1):iTrans)], tol=1e-4) 
                #Rnorm(gene[c((uTrans+1):iTrans)])
                pBNew <- Rnorm.exp(gene[c((DTe+1):NROW(gene))], tol=1e-4) 

                PSI <- (pPNew$parameters[1])*dnorm(c(min(gene):max(gene)), 
                        pPNew$parameters[2], pPNew$parameters[3])+
                        (1-pPNew$parameters[1])*dexp(c(min(gene):max(gene)), 
                        1/pPNew$parameters[4]) ## 1/rate
                BAS <- (pBNew$parameters[1])*dnorm(c(min(gene):max(gene)), 
                        pBNew$parameters[2], pBNew$parameters[3])+
                        (1-pBNew$parameters[1])*dexp(c(min(gene):max(gene)), 
                        1/pBNew$parameters[4]) ## 1/rate
            }
            else if(emissionDistAssumption == "gamma") {
                ## Refit and calculate KL-divergence at that point.
                pPNew <- RgammaMLE(gene[c((DTs+1):DTe)])
                pBNew <- RgammaMLE(gene[c((DTe+1):NROW(gene))])

                ## Estimate KL-divergence.
                PSI <- dgamma(c(min(gene):max(gene)), shape=pPNew$shape, 
                    scale=pPNew$scale)
                BAS <- dgamma(c(min(gene):max(gene)), pBNew$shape, pBNew$scale)
            }
            minD2 <- 1e-300
            KLdivHMM <- sum((PSI*log((PSI+minD2)/(BAS+minD2))))

            ANS[i] <- (DTe-DTs)*size
            STRTwave[i] <- DTs*size
            ENDwave[i] <- DTe*size
            KLdivFinal[i] <- KLdiv[DTe] 
            ##KLdivHMM## Try switching to the empirical KL-div.
            KLdivPar[i] <- KLdivHMM

            ## Calculates min/max and min/avg filters.
            medDns <- median(MovMax[c(max((which(ansVitervi == 1))+
                    round(MovMeanSpd/size)):NROW(MovMax))])
            minMax <- min(MovMax[c(min(which(ansVitervi == 1))
                    :max(which(ansVitervi == 1)))])
            minWindLTMed[i] <- (medDns < minMax) 
            ## True if min(wave) > med(wave.upstream)
            avgDns <- median(MovMean[c(max((which(ansVitervi == 1))+
                    round(MovMeanSpd/size)):NROW(MovMean))])
            minAvg <- min(MovMean[c(min(which(ansVitervi == 1))
                    :max(which(ansVitervi == 1)))])
            minMeanWindLTMed[i] <- (avgDns < minAvg) 
            ## True if min(wave) > med(wave.upstream)
            
        ## Construct the return value...
        geneData[[1]] <- gene[c(1:DTs)]
        geneData[[2]] <- gene[c((DTs+1):DTe)]
        geneData[[3]] <- gene[c((DTe+1):NROW(gene))]
        geneData[[4]] <- emis1 ## Value of the gene... c1.
        geneData[[5]] <- emis2 ## Value of the gene... c2.
        geneData[[6]] <- ans[[4]] ## Matrix of posteriors.
        geneData[[7]] <- ans[[5]] 
        ## Posteriors of a transition from state 2->3.        
        geneData[[8]] <- unmap 
        dataList[[i]] <- geneData
        }
    }
    
    returnDF <- data.frame(StartWave= STRTwave, EndWave= ENDwave, Rate= ANS, 
        KLdiv= KLdivFinal, KLdivParametric= KLdivPar, minOfMax= minWindLTMed, 
        minOfAvg= minMeanWindLTMed, ID= genes[,5], ExternalID= genes[,6])

    if(returnVal == "simple") {
     return(returnDF)
    }
    if(returnVal == "alldata") {
     dataList[[NROW(genes)+1]] <- returnDF
     return(dataList)
    }
} 

###########################################################################
##
##   Copyright 2013, 2014 Charles Danko and Minho Chae.
##
##   This program is part of the groHMM R package
##
##   groHMM is free software: you can redistribute it and/or modify it
##   under the terms of the GNU General Public License as published by
##   the Free Software Foundation, either version 3 of the License, or
##   (at your option) any later version.
##
##   This program is distributed in the hope that it will be useful, but
##   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
##   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
##   for more details.
##
##   You should have received a copy of the GNU General Public License along
##   with this program.  If not, see <http://www.gnu.org/licenses/>.
##
##########################################################################

################################################################################
##
##  4/24/2012
##
##  Thanks to Andre Martins for the following two functions, useful for 
##  calculating the confidence interval for a ratio of Poisson random variables.
## 
##  Based on: Ederer F, Mantel N (1974); Confidence Limits on the Ratio of Two 
## Poisson Variables.  AMERICAN JOURNAL OP EPIDEMIOLOGY.
##
############################################################################### 
approx.ratio.CI <- function(x1, x2, alpha=0.05) {
  t = qnorm(1 - alpha/2)
  n = x1 + x2
  zp = (t^2/2 + x1 + 1/2)^2 - ((n + t^2)/n) * (x1 + 1/2)^2
  zn = (t^2/2 + x1 - 1/2)^2 - ((n + t^2)/n) * (x1 - 1/2)^2
  
  a = (t^2/2 + x1 + 1/2 + sqrt(zp)) / (n + t^2/2 - x1 - 1/2 - sqrt(zp))
  b = (t^2/2 + x1 - 1/2 - sqrt(zn)) / (n + t^2/2 - x1 + 1/2 + sqrt(zn))

  return(c(b, a))
}

approx.ratios.CI <- function(num.counts, denom.counts, alpha=0.05) {
  stopifnot(length(num.counts) == length(denom.counts))
  N = length(num.counts)

  result = matrix(data=0, nrow=N, ncol=2)

  for (i in 1:N)
    result[i, ] = approx.ratio.CI(num.counts[i], denom.counts[i], alpha)

  return(result)
}

#' Returns the pausing index for different genes.  TODO: DESCRIBE THE PAUSING
#' INDEX.
#'
#' Supports parallel processing using mclapply in the 'parallel' package.
#' To change the number of processors, use the argument 'mc.cores'.
#'
#' @param features A GRanges object representing a set of genomic coordinates.
#' @param reads A GRanges object representing a set of mapped reads.
#' @param size The size of the moving window.
#' @param up Distance upstream of each f to align and histogram.
#' @param down Distance downstream of each f to align and histogram (NULL).
#' @param UnMAQ Data structure representing the coordinates of all un-mappable
#'   regions in the genome.
#' @param debug If set to TRUE, provides additional print options.  Default:
#'   FALSE
#' @param ... Extra argument passed to mclapply
#' @return Returns a data.frame of the pausing indices for the input genes.
#' @author Charles G. Danko and Minho Chae.
#' @return Returns the pausing index for different genes.
#' @examples
#' features <- GRanges("chr7", IRanges(2394474,2420377), strand="+")
#' reads <- as(readGAlignments(system.file("extdata", "S0mR1.bam",
#'             package="groHMM")), "GRanges")
#' ## Not run:
#' # pi <- pausingIndex(features, reads)
##
##  Arguments:
##  f   -> data.frame of: CHR, START, END, STRAND.
##  p   -> data.frame of: CHR, START, END, STRAND.
##  size    -> The size of the moving window.
##  up  -> Distance upstream of each f to align and histogram.
##  down    -> Distance downstream of each f to align and histogram (NULL).
##  UnMAQ   -> Vector of integers representing the coordinates of all 
##  un-MAQable regions in the genome.
##
##  Assumptions:
##  (1) 
##
##  TODO: 
##  (1) Write C function. 
##  (2) Implement: ...
##      (a)  promWind-> As an alternative to up/down, specify a specific window
##      in which to check for the pause-peak.
##      Uses the same format as "f", expect that features are mapped with 
##      respect to the start of f, where 0 indicates the start of 
##      transcription, and negative numbers specify upstream sequence.
##      This is likely useful for rate; perhaps for identifying internal 
##      paused-peaks...
pausingIndex <- function(features, reads, size=50, up=1000, down=1000, 
    UnMAQ=NULL, debug=FALSE, ...) {
    # make sure reads are sorted
    reads <- reads[order(as.character(seqnames(reads)), start(reads)), ]
    f <- data.frame(chrom=as.character(seqnames(features)), 
        start=as.integer(start(features)), end=as.integer(end(features)), 
        strand=as.character(strand(features)))
    if ("symbol" %in% names(mcols(features))){
        f <- cbind(f, symbol=features$symbol) 
    } else {
        f <- cbind(f, symbol=GeneID <- as.character(seq_len(NROW(f))))
    }
    p <- data.frame(chrom=as.character(seqnames(reads)), 
        start=as.integer(start(reads)), end=as.integer(end(reads)), 
        strand=as.character(strand(reads)))

    C <- sort(as.character(unique(f[[1]])))
    Pause <- rep(0,NROW(f))
    Body  <- rep(0,NROW(f))
    Fish  <- rep(0,NROW(f))
    GeneID <- rep("", NROW(f))
    CIl  <- rep(0,NROW(f))
    CIu  <- rep(0,NROW(f))

    ## Pass back information for the fisher test...
    PauseCounts <- rep(0, NROW(f))
    BodyCounts  <- rep(0, NROW(f))
    UpCounts    <- rep(0, NROW(f))
    UgCounts    <- rep(0, NROW(f))

    size        <- as.integer(size)
    up          <- as.integer(up)
    down        <- as.integer(down)

    ###### Calculate PLUS and MINUS index, for DRY compliance.
    PLUS_INDX <- which(f[[4]] == "+")
    MINU_INDX <- which(f[[4]] == "-")

    ###### Identify TSS -- Start for '+' strand, End for '-' strand.
    if(debug) {
        message("Calculating TSS and gene ends for each gene based 
            on strand information.")
    }
    c_tss_indx <- rep(0,NROW(f))
    c_tss_indx[PLUS_INDX] <- 2
    c_tss_indx[MINU_INDX] <- 3
    c_tss <- unlist(lapply(c(1:NROW(f)), function(x) { f[x, c_tss_indx[x]] }))

    ###### Now calculate left and right position for gene body, based 
    ### on '+' or '-'.
    ### Calculate gene end.  Gene start is contiguous with the coordinates 
    ###for the promoter.
    c_gene_end_indx <- rep(0,NROW(f))
    c_gene_end_indx[PLUS_INDX] <- 3
    c_gene_end_indx[MINU_INDX] <- 2
    c_gene_end <- unlist(lapply(c(1:NROW(f)), function(x) { 
                f[x,c_gene_end_indx[x]] }))

    ### Assign left and right.
    gLEFT   <- rep(0,NROW(c_tss))
    gRIGHT  <- rep(0,NROW(c_tss))
    gLEFT[PLUS_INDX]    <- c_tss[PLUS_INDX] + down
    gRIGHT[PLUS_INDX]   <- c_gene_end[PLUS_INDX]
    gLEFT[MINU_INDX]    <- c_gene_end[MINU_INDX]
    gRIGHT[MINU_INDX]   <- c_tss[MINU_INDX] - down

    ## Run parallel version.
    mcp <- mclapply(c(1:NROW(C)), pausingIndex_foreachChrom, C=C, f=f, p=p, 
                    gLEFT=gLEFT, gRIGHT=gRIGHT, c_tss=c_tss, 
                    size=size, up=up, down=down, UnMAQ=UnMAQ, debug=debug, ...)
   
    ## Unlist and re-order values for printing in a nice data.frame.
    for(i in 1:NROW(C)) {
        # Which KG?  prb?
        indxF   <- which(as.character(f[[1]]) == C[i])
                indxPrb <- which(as.character(p[[1]]) == C[i])

        if((NROW(indxF) >0) & (NROW(indxPrb) >0)) {
            Pause[indxF][mcp[[i]][["ord"]]]     <- mcp[[i]][["Pause"]]
            Body[indxF][mcp[[i]][["ord"]]]  <- mcp[[i]][["Body"]]
            Fish[indxF][mcp[[i]][["ord"]]]  <- mcp[[i]][["Fish"]]
            GeneID[indxF][mcp[[i]][["ord"]]]    <- mcp[[i]][["GeneID"]]
            
            PauseCounts[indxF][mcp[[i]][["ord"]]] <- mcp[[i]][["PauseCounts"]]
            BodyCounts[indxF][mcp[[i]][["ord"]]]  <- mcp[[i]][["BodyCounts"]]
            UpCounts[indxF][mcp[[i]][["ord"]]]    <- mcp[[i]][["UpCounts"]]
            UgCounts[indxF][mcp[[i]][["ord"]]]    <- mcp[[i]][["UgCounts"]]

            CIl[indxF][mcp[[i]][["ord"]]] <- mcp[[i]][["CIl"]]
            CIu[indxF][mcp[[i]][["ord"]]] <- mcp[[i]][["CIu"]]
        }
    }

    return(data.frame(Pause= Pause, Body= Body, Fisher= Fish, GeneID= GeneID, 
        CIlower=CIl, CIupper=CIu, PauseCounts= PauseCounts, 
        BodyCounts= BodyCounts, uPCounts= UpCounts, uGCounts= UgCounts))
}

pausingIndex_foreachChrom <- function(i, C, f, p, gLEFT, gRIGHT, c_tss, size, 
    up, down, UnMAQ, debug) {
        if(debug) {
            message(C[i])
        }

        # Which KG?  prb?
        indxF   <- which(as.character(f[[1]]) == C[i])
        indxPrb <- which(as.character(p[[1]]) == C[i])

        if((NROW(indxF) >0) & (NROW(indxPrb) >0)) {
            # Order -- Make sure, b/c this is one of our main assumptions.  
            # Otherwise violated for DBTSS.
            Ford <- order(f[indxF,2])
            Pord <- order(p[indxPrb,2])

            # Type coersions.
            FeatureStart    <- as.integer(gLEFT[indxF][Ford])
            FeatureEnd  <- as.integer(gRIGHT[indxF][Ford])
            FeatureStr  <- as.character(f[indxF,4][Ford])
            FeatureTSS  <- as.integer(c_tss[indxF][Ford])

            PROBEStart  <- as.integer(p[indxPrb,2][Pord])
            PROBEEnd    <- as.integer(p[indxPrb,3][Pord])
            PROBEStr    <- as.character(p[indxPrb,4][Pord])

            # Set dimensions.
            dim(FeatureStart)   <- c(NROW(FeatureStart), NCOL(FeatureStart))
            dim(FeatureEnd)     <- c(NROW(FeatureEnd),   NCOL(FeatureEnd))
            dim(FeatureTSS)     <- c(NROW(FeatureTSS),   NCOL(FeatureTSS))
            dim(FeatureStr)     <- c(NROW(FeatureStr),   NCOL(FeatureStr))
            dim(PROBEStart)     <- c(NROW(PROBEStart),   NCOL(PROBEStart))
            dim(PROBEEnd)       <- c(NROW(PROBEEnd),     NCOL(PROBEEnd))
            dim(PROBEStr)       <- c(NROW(PROBEStr),     NCOL(PROBEStr))

        ## Run the calculations on the gene.
            ## Calculate the maximal 50 bp window.
            if(debug) {
                message(C[i],": Counting reads in pause peak.")
            }
            HPause <- .Call("NumberOfReadsInMaximalSlidingWindow", 
                        FeatureTSS, FeatureStr, PROBEStart, PROBEEnd, 
                        PROBEStr, size, up, down, PACKAGE = "groHMM")

            ## Run the calculate on the gene body...
            if(debug) {
                message(C[i],": Counting reads in gene.")
            }
            HGeneBody <- .Call("CountReadsInFeatures", FeatureStart, 
                FeatureEnd, FeatureStr, PROBEStart, PROBEEnd, PROBEStr, 
                PACKAGE = "groHMM")

        ## Get size of gene body
            Difference <- FeatureEnd-FeatureStart
            Difference[Difference < 0] <- 0 
            ## Genes < 1kb, there is no suitable area in the body of the gene.

        ## Calculate UN-MAQable regions...
            if(!is.null(UnMAQ)) {

                ## Count start index.
                chr_indx <- which(UnMAQ[[1]][[1]] == C[i])
                CHRSIZE <- as.integer(UnMAQ[[1]][[2]][chr_indx])
                CHRSTART <- as.integer(0)
                if(chr_indx > 1) {  ## Running on 1:0 gives c(1, 0)
                    CHRSTART <- as.integer( 
                        sum(UnMAQ[[1]][[2]][
                            c(1:(chr_indx-1))
                        ]) +1)
                }

                if(debug) {
                    message(C[i],": Counting unMAQable regions.")
                    message("CHRSIZE:", CHRSIZE, "CHRSTART:", CHRSTART)
                }
                
                ## Count unMAQable regions, and size of everything ... 
                nonmappable <- .Call("CountUnMAQableReads", 
                    FeatureStart, FeatureEnd, as.integer(UnMAQ[[2]]), 
                    CHRSTART, CHRSIZE, PACKAGE = "groHMM")

                ## Adjust size of gene body.
                Difference <- Difference - nonmappable + 1 
                ## Otherwise, get -1 for some.

                if(debug) {
                    print(head(nonmappable))
                    print(as.integer(head(Difference)))
                }
            }

        ## Now use Fisher's Exact.
            if(debug) {
                message(C[i],": Using Fisher's exact.")
            }
            # Make uniform reads.
            Up <- round(HPause + HGeneBody)*(size)/(size+Difference) 
            ## Expted in pause == 
            ##((total reads)/ (total size) [reads/ base]) * 
            ## size [reads/ pause window]
            Ug <- round(HPause + HGeneBody)*(Difference)/(size+Difference) 
            ## Expted reads in body == 
            ## ((total reads)/ (total size) [reads/ base]) * 
            ## (gene size) [reads/ gene body]
            HFish <- unlist(lapply(c(1:NROW(Ford)), function(x) {
               fisher.test(
                  data.frame(
                c(HPause[x], round(Up[x])), 
                c(HGeneBody[x], round(Ug[x]))
                  )
               )$p.value
            } ))

        ## Make return values.
            Pause_c     <- as.double(HPause/size)
            Body_c  <- as.double((HGeneBody+1)/Difference) 
            ## 6-5-2012 -- Add a pseudocount here, 
            ## forcing at least 1 read in the gene body.   
            Fish_c  <- as.double(HFish)
            GeneID_c    <- as.character(f[indxF,5][Ford])
            
            PauseCounts_c <- HPause
            BodyCounts_c  <- HGeneBody
            UpCounts_c    <- round(Up)
            UgCounts_c    <- round(Ug)

            aCI <- approx.ratios.CI(HPause, HGeneBody)
            scaleFactor <- Difference/size ## Body/ pause, must be 1/ PI units.
            CIl_c <- as.double(aCI[,1]*scaleFactor)
            CIu_c <- as.double(aCI[,2]*scaleFactor)
            
            return(list(Pause= Pause_c, Body= Body_c, Fish= Fish_c, 
                GeneID= GeneID_c, PauseCounts= PauseCounts_c, 
                BodyCounts= BodyCounts_c, UpCounts= UpCounts_c, 
                UgCounts= UgCounts_c, CIl= CIl_c, CIu= CIu_c, ord= Ford))
        }
    return(integer(0))
}

%\VignetteIndexEntry{groHMM tutorial}
%\VignettePackage{groHMM}
%\VignetteEngine{utils::Sweave}

\documentclass{article}

\usepackage{subfig, graphicx}

\newcommand{\Rmethod}[1]{{\texttt{#1}}}
\newcommand{\Rfunarg}[1]{{\texttt{#1}}}

\newcommand{\groHMM}{\Rpackage{groHMM}}

<<style, eval=TRUE, echo=FALSE, results=tex>>=
BiocStyle::latex(use.unsrturl=FALSE)
savewd = getwd()
@

\begin{document}
\SweaveOpts{concordance=TRUE}
\SweaveOpts{prefix.string=\Sexpr{savewd}/groHMM}

\newcommand{\exitem}[3]{\item \Rcode{\textbackslash#1\{#2\}} #3 \csname#1\endcsname{#2}.}

\title{\Rpackage{groHMM} Tutorial}
\author{
\author[1,2]{Minho Chae}
\author[3]{ Charles G. Danko}
\author[1,2]{W. Lee Kraus\thanks{\email{lee.kraus@utsouthwestern.edu}}}
\affil[1]{Laboratory of Signaling and Gene Regulation,Division of Basic Sciences, Department of Obstetrics and Gynecology}
\affil[2]{Cecil H. \& Ida Green Center for Reproductive Biology Science, UT Southwestern Medical Center, Dallas, Texas, 75390-8511}
\affil[3]{Baker Institute for Animal Health, College of Veterinary Medicine,Cornell University, Ithaca, NY 14853}
}

\date{\today}

\maketitle

\tableofcontents

\section{Introduction}
\underline{G}lobal Nuclear \underline{R}un \underline{O}n and Sequencing
(GRO-seq) was developed for comprehensively map transcriptional activity in
cells \cite{Core2008,Hah2011}.  GRO-seq, which provides a genome wide `map'
of the position and orientation of all transcriptionally active RNA
polymerases, has become increasingly widely used in recent years because it
has numerous advantages compared to alternative methods of
transcriptome profiling, such as expression microarrays and RNA-seq.
Among these, GRO-seq provides information on instantaneous transactional
responses because it detects primary transcription, as opposed to mature,
processed mRNA.  In addition, because it is independent of RNA polyadenylation,
processing, and stability, GRO-seq provides extensive information on the
non-coding transcriptome, including primary miRNAs, lincRNAs, enhancer RNAs,
and potentially additional, yet undiscovered classes of transcription
occurring in cells \cite{Hah2011,Hah2013,Luo2014}.
Thus, GRO-seq data provides a complete and instantaneous picture of
transcription, and has extensive applications in deciphering the mechanisms
of transcriptional regulation.

We have recently developed several important analytical approaches which make
use of GRO-seq data to address new biological questions.
Our pipleline has been packaged and documented, resulting in
the \Rpackage{groHMM} package for Bioconductor.
Among the more advanced features, \Rpackage{groHMM} predicts the boundaries of
transcriptional activity across the genome \emph{de novo} using a two-state
hidden Markov model (HMM).
Our model essentially divides the genome into ``transcribed'' and
``non-transcribed'' regions in a strand specific manner \cite{Hah2011}.
We also use HMMs to identify the leading edge of Pol II at genes activated by a
stimulus in GRO-seq time course data.  This approach allows the genome-wide
interrogation of transcription rates in cells \cite{Danko2013}.

In addition to these advanced features, \Rpackage{groHMM} provides wrapper
functions for counting raw reads \cite{PagesGRanges}, generating wiggle
files for visualization \cite{Michael2009}, and creating metagene
(averaging) plots.  \Rpackage{groHMM} takes over all aspects of
analysis after reads have been aligned to a reference genome with
short-read alignment tools.  Although \Rpackage{groHMM} is tailored
towards GRO-seq data, the same functions and analytical methodologies
can, in principal, be applied to a wide variety of other short read data
sets since the package includes a number of easily usable and extensible
functions for general short read data analysis.
This guide focuses on the most common application of the package.


\section{Preparation}
The \Rpackage{groHMM} package is available in the Bioconductor and can be
downloaded as follows:
<<install groHMM, eval=FALSE>>=
if (!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")
BiocManager::install("groHMM")

@

The following packages are not required to use \Rpackage{groHMM}, but they
are used to download annotations and evaluate transcripts, and should be
installed for this tutorial.
<<install packages, eval=FALSE>>=
BiocManager::install("GenomicFeatures")
BiocManager::install("org.Hs.eg.db")
BiocManager::install("edgeR")
BiocManager::install("TxDb.Hsapiens.UCSC.hg19.knownGene")
@

\section{\Rpackage{groHMM} Workflow}
\subsection{Read GRO-seq Data Files}
In this tutorial we will use example data from Hah et al. [2011] (GEO
accession GSE27463).   This experiment was designed to assess transcriptional
changes following treatment of MCF-7 cells with 17$\beta$ estradiol (E2).
The data include a time course of GRO-seq data following treatment with E2
(\emph{i.e.}, 0, 10, 40 and 160 min.).  Two biological replicates are
available for each time point.  Bed files were obtained from GEO and lifted
over to hg19 using the UCSC \Rpackage{liftOver} tool.
In order to make the package size more manageable, we have included
data from chromosome 7 only in 0 and 10 min. conditions.
\Rpackage{groHMM} supports parallel processing and the number of cores to
use can be set using \Rcode{mc.cores} option.

<<use groHMM, eval=TRUE>>=
library(groHMM)
options(mc.cores=getCores(4))
@

\Rpackage{groHMM} uses the \Rpackage{GRanges} class from the
\Rpackage{GenomicRanges} packages to represent a collection of genomic
features, allowing synergy with other useful packages in Bioconductor.
Most of the functions in the package take at least two arguments: `reads'
and `features'.  Reads represent the genomic coordinates of a
set of mapped short reads.   Features represent a set of genomic coordinates
of interest such as genes, exons, or transcripts.

The example data included in this package can be loaded into \software{R}
using the following commands.

<<load data, eval=TRUE>>=
S0mR1 <- as(readGAlignments(system.file("extdata", "S0mR1.bam",
        package="groHMM")), "GRanges")
S0mR2 <- as(readGAlignments(system.file("extdata", "S0mR1.bam",
        package="groHMM")), "GRanges") # Use R1 as R2
S40mR1 <- as(readGAlignments(system.file("extdata", "S40mR1.bam",
        package="groHMM")), "GRanges")
S40mR2 <- as(readGAlignments(system.file("extdata", "S40mR1.bam",
        package="groHMM")), "GRanges") # Use R1 as R2
@


\subsection{Create a Wiggle File}
Wiggle files are created for each strand after replicates are combined in
order to visualize GRO-seq data in the UCSC genome browser.
Wiggle files can be also normalized by the sequencing depth, \emph{i.e.},
average number of reads in the dataset. \Rcode{writeWiggle} function is a
wrapper of \Rcode{export} in \Rpackage{rtracklayer} for generation of
wiggle/bigWig type of files.

<<combine replicates, eval=TRUE>>=
# Combine replicates
S0m <- c(S0mR1, S0mR2)
S40m <- c(S40mR1, S40mR2)
@

<<write wiggle files, eval=FALSE>>=
writeWiggle(reads=S0m, file="S0m_Plus.wig", fileType="wig", strand="+",
        reverse=FALSE)
writeWiggle(reads=S0m, file="S0m_Minus.wig", fileType="wig", strand="-",
        reverse=TRUE)

# For BigWig file:
# library(BSgenome.Hsapiens.UCSC.hg19)
# si <- seqinfo(BSgenome.Hsapiens.UCSC.hg19)
# writeWiggle(reads=S0m, file="S0m_Plus.wig", fileType="BigWig", strand="+",#
#               reverse=FALSE, seqinfo=si)

# Normalized wiggle files
expCounts <- mean(c(NROW(S0m), NROW(S40m)))
writeWiggle(reads=S0m, file="S0m_Plus_Norm.wig", fileType="wig", strand="+",
           normCounts=expCounts/NROW(S0m), reverse=FALSE)
@

The resulting wiggle files can be uploaded as `custom tracks' in the UCSC
genome browser, or your visualization software of choice.



\subsection{Transcript Calling}

\begin{figure}[h]
    \centering
    \subfloat[GRO-seq with called transcripts]{\label{FigGROseq}
    \includegraphics[width=.55\textwidth]{GROseqData.png}}
    \qquad
    \subfloat[2-state HMM]{\label{FigHMM}
    \includegraphics[width=.35\textwidth]{GROseqHMM.png}}
    \caption{HMM calling of GRO-seq data}
\end{figure}

In \Rpackage{groHMM}, transcribed regions are detected \emph{de novo} using
a two-state hidden Markov model (HMM).
The model takes GRO-seq read counts as input across the
genome and divides the genome into ``transcribed'' and ``non-transcribed''
state as shown in Figure 1.  First, a single read set is generated by
combining all samples for each time point.  This combined approach improves
sensitivity for transcripts with low expression levels.
Combined reads are used to train the model parameters using the
Baum-Welch Expectation Maximization (EM) algorithm.  Each strand is modeled
separately dividing the genome into non-overlapping 50 bp windows classified
as either state.


<<transcript calling, eval=TRUE, keep.source=TRUE, results=hide>>=
Sall <- sort(c(S0m, S40m))
# hmmResult <- detectTranscripts(Sall, LtProbB=-200, UTS=5,
#                   threshold=1)
# Load hmmResult from the saved previous run
load(system.file("extdata", "Vignette.RData", package="groHMM"))
txHMM <- hmmResult$transcripts
@
<<called transcripts, eval=TRUE, keep.source=TRUE>>=
head(txHMM)
@

The \Rfunction{detectTranscripts} function also uses two hold-out parameters.
These parameters, specified by the arguments \Rfunarg{LtProbB} and
\Rfunarg{UTS}, represents the log-transformed transition probability of
switching from transcribed state to non-transcribed state and variance of the
emission probability for reads in the non-transcribed state, respectively.
Holdout parameters are used to optimize the performance of HMM predictions
on known genes.


\subsection{Evaluation of Transcript Calling}
Predicted transcripts are evaluated by comparison to known annotations, making
the assumption that GRO-seq transcripts should largely be in agreement with
available annotations.  Two types of error may occur, as described
below.  The HMM parameters are evaluated by the sum of the error rates.
The procedure involves collecting a set of high-confidence reference
transcripts.  An annotation dataset can be constructed by downloading
from the UCSC database or alternatively, pre-made TranscriptDb objects can be
used if they are available in the Bioconductor.  We will use the UCSC knownGene
track and retrieve transcript annotations with
\Rpackage{GenomicFeatures} \cite{CarsonGFeatures} package.

<<use ucsc genes, eval=TRUE>>=
library(TxDb.Hsapiens.UCSC.hg19.knownGene)
kgdb <- TxDb.Hsapiens.UCSC.hg19.knownGene

library(GenomicFeatures)
# For refseq annotations:
# rgdb <- makeTxDbFromUCSC(genome="hg19", tablename="refGene")
# saveDb(hg19RGdb, file="hg19RefGene.sqlite")
# rgdb <- loadDb("hg19RefGene.sqlite")
kgChr7 <- transcripts(kgdb, filter=list(tx_chrom = "chr7"),
                      columns=c("gene_id", "tx_id", "tx_name"))
seqlevels(kgChr7) <- seqlevelsInUse(kgChr7)
@

Because annotations do not provide precise cell type-specific expression
information, overlapping transcripts must be merged into a single set,
in which each annotation represents the 5$'$- and 3$'$-most boundaries of
genes.  Different isoforms of each gene are collapsed into one using the
ENTREZID.  These consensus annotations are used for the evaluation of HMM
calling.

<<make consensus annotations, eval=TRUE, keep.source=TRUE, results=hide>>=
# Collapse overlapping annotations
kgConsensus <- makeConsensusAnnotations(kgChr7, keytype="gene_id",
        mc.cores=getOption("mc.cores"))
library(org.Hs.eg.db)
map <- select(org.Hs.eg.db,
    keys=unlist(mcols(kgConsensus)$gene_id),
        columns=c("SYMBOL"), keytype=c("ENTREZID"))
mcols(kgConsensus)$symbol <- map$SYMBOL
mcols(kgConsensus)$type <- "gene"
@

There are two types of error that can be evaluated when comparing predicted
transcripts with annotations.  (1) The number of transcripts
overlapping two or more annotations, (\emph{i.e.}, these transcripts `merged annotations
together') and (2) the number of annotations that overlap two or more
transcripts on the same strand (\emph{i.e.}, we say that these transcript
calls `dissociated a single annotation') must be determined.   The optimal tuning
parameters can be found by minimizing the sum of the two errors.
This approach allows identification of the model that best fits
the existing annotations and should more precisely predict transcripts
in non-annotated parts of the genome.
<<evaluation of transcript calling, eval=TRUE>>=
e <- evaluateHMMInAnnotations(txHMM, kgConsensus)
e$eval
@

\subsection{HMM Tuning}
Here we demonstrate how the optimal value for each tuning parameters can be
obtained by running HMM multiple times over a certain range of the parameters.
Among the nine test cases, both the sum of errors and error rate per called transcript
show minimal at case \#7, as shown below.  The variation of errors should be
greater if whole chromosomes are used.
Also, a larger set of the parameters might be used in practice.  This tunning
step takes long time, so you may skip it for quick review of the package.

<<tuning HMM, eval=TRUE, results=hide>>=
tune <- data.frame(LtProbB=c(rep(-100,3), rep(-200,3), rep(-300,3)),
           UTS=rep(c(5,10,15), 3))

Fp <- windowAnalysis(Sall, strand="+", windowSize=50)
Fm <- windowAnalysis(Sall, strand="-", windowSize=50)

# evals <- mclapply(seq_len(9), function(x) {
#        hmm <- detectTranscripts(Fp=Fp, Fm=Fm, LtProbB=tune$LtProbB[x],
#                  UTS=tune$UTS[x])
#        e <- evaluateHMMInAnnotations(hmm$transcripts, kgConsensus)
#        e$eval
#        }, mc.cores=getOption("mc.cores"),  mc.silent=TRUE)
tune <- cbind(tune, do.call(rbind, evals))  # evals from the previous run
@

<<check tuning, eval=TRUE>>=
tune
which.min(tune$total)
which.min(tune$errorRate)
@

To robustly compare transcripts with known genes, densities representing the
frequency of transcripts can be calculated relative to their mapped gene
annotations.
Conceptually, the plot is divided into three distinct regions as shown in
Figure 2, including upstream of known gene annotations, inside genes,
and downstream of the annotated polyadenylation site.

Metrics to evaluate the degree of overlap with gene annotations focus on the
region upstream of gene annotations, which provides a measure of specificity,
and the region inside of genes, which provides a measure of sensitivity.
The region downstream of the polyadenylation site is known to contain residual
transcription \cite{Core2008} and consequently is not used to define quality.

The metrics are defined relative to an `ideal' transcript caller, which takes
the form of a step function (\emph{i.e.}, red line in the plot).
Our quality metrics represent the following
(see the plot below for a graphical representation):

\begin{enumerate}
  \item true positive; gene body (TP)  =
    (gene annotation area under the curve) /(max area for matched transcripts),
  \item false negative; gene body (FN) =  1 - TP,
  \item 5$'$ false positive; upstream region (5$'$FP)  =
    (5$'$ overhang area under the curve) /(max area for matched transcripts),
  \item 5$'$ true negative; upstream region (5$'$TN) = TP - 5$'$FP.
\end{enumerate}

We constrained 5$'$FP and 5$'$TN so that their sum to be TP in order to
use the upstream region only if positive number of transcipts are called in the gene body
for the calucation of the quality metrics.
Note that these quality metrics are conceptually very similar to true
positive, true negative, false positive and false negative, respectively.
During the comparison, only expressed annotations are used
and genes either too small or too large in size are excluded.
And also size of transcripts and annotations are
scaled to a smaller unit, \emph{i.e.}, 1K for a visual representation.
Here best overlapped annotations or transcripts
are used for either `merged annotations' or  `dissociated a single annotation'
type of error.  Final quality metrics are represented as TUA (Transcription
Unit Accuracy).

<<expressed genes, eval=TRUE>>=
getExpressedAnnotations <- function(features, reads) {
     fLimit <- limitToXkb(features)
     count <- countOverlaps(fLimit, reads)
     features <- features[count!=0,]
     return(features[(quantile(width(features), .05) < width(features))
    & (width(features) < quantile(width(features), .95)),])}
conExpressed <- getExpressedAnnotations(features=kgConsensus,reads=Sall)
@
<<fig2plot, eval=TRUE>>=
td <- getTxDensity(txHMM, conExpressed, mc.cores=getOption("mc.cores"))
u <- par("usr")
lines(c(u[1], 0, 0, 1000, 1000, u[2]), c(0,0,u[4]-.04,u[4]-.04,0,0),
      col="red")
legend("topright", lty=c(1,1), col=c(2,1), c("ideal", "groHMM"))
text(c(-500,500), c(.05,.5), c("FivePrimeFP", "TP"))
td
@


\begin{figure}
\begin{center}
<<label=fig2,fig=TRUE,echo=FALSE, results=hide>>=
<<fig2plot>>
@
\end{center}
\caption{Transcript Density Plot}
\label{Figure:2}
\end{figure}

\subsection{Working with non-mammalian Genomes}
If your target of study is non-mammalian, you can retrieve the relevant annotations
from the Bioconductor if they are supported.  You can check the availability with
the function \Rfunction{supportedUCSCtable} in \Rpackage{GenomicsFeautes}.
If the organism is not supported, you can still build consensus annotations by
directly downloading the annotation table for the organism from the UCSC genome browser.
The following command line shows the mysql query in linux to download protein-coding RefSeq genes
for all chromosomes except chrM for C. Elegans saving into refgene.bed file.
You can find more information about using MySQL for the UCSC genome browser at
https://genome.ucsc.edu/goldenPath/help/mysql.html.

<<Non-mammalian, eval=FALSE, keep.source=TRUE, results=hide>>=
# mysql --user=genome --host=genome-mysql.cse.ucsc.edu ce10 -e \
#  "select chrom, txStart, txEnd, name, exonCount, strand, name2 from refGene \
#   where chrom not like chrom!='chrM' and cdsStart != cdsEnd" | tail -n +1 > refgene.bed

# G <- read.table("refgene.bed", header=TRUE, stringsAsFactors=FALSE, sep="\t")
# ce <- GRanges(G$chrom, IRanges(G$txStart, G$txEnd), strand=G$strand, \
#               access=G$name, gene_id=G$name2)
# ceConsensus <- makeConsensusAnnotations(ce, keytype="gene_id", \
#                mc.cores=getOption("mc.cores"))
@

As for transcript calling, the default values for \Rfunction{detectTranscripts} were set
for mammalian genome.  In case of C. Elegans genome, it is much smaller than human genome and
the genes are more tightly located.  So we recommend to explore higher values
for the transition probability from the transcribed to non-transcribed state.
For example, you can use i.e., values >-50 instead of -200 for \Rcode{LtProbB}.


\subsection{Repairing Transcript Calling with Annotations}
Prediction of transcripts by the HMM is not perfect.
Discrepancies with the annotations will occur even after the parameters
are optimally tuned.  Transcript calls can be `fixed' for known types
of error by (1) breaking transcripts that have merged annotations and
(2) combining transcripts that have dissociated a single annotation.
The following method will generate a final set of transcripts for further analysis.

<<repairing called transcripts, eval=TRUE>>=
bPlus <- breakTranscriptsOnGenes(txHMM, kgConsensus, strand="+")
bMinus <- breakTranscriptsOnGenes(txHMM, kgConsensus, strand="-")
txBroken <- c(bPlus, bMinus)
txFinal <- combineTranscripts(txBroken, kgConsensus)
@

<<plotTxDensity_final, eval=FALSE, keep.source=TRUE, results=hide>>=
tdFinal <- getTxDensity(txFinal, conExpressed, mc.cores=getOption("mc.cores"))
@

\subsection{Differential Analysis with \Rpackage{edgeR}}
There are several packages in Bioconductor for differential expression
analysis such as \Rpackage{DESeq}, \Rpackage{baySeq}, \Rpackage{DEGSeq},
or \Rpackage{edgeR}.
\Rpackage{edgeR} \cite{Robinson2010} is used for this demonstration.
Differential expression analysis can be done using either by called
transcripts or known annotations depending on the nature of your research
question.
The procedures are quite similar except reads are counted using the
genomic locations of either called transcript or known annotations.   For
longer transcripts or annotated genes, we use a window of +1 to +13 kb from
the transcription start site (TSS), which was chosen in order to exclude reads
from RNA polymerases engaged at the promoter and to allow enough time for the
elongation of newly initiated Pol II (See Hah et al., 2011).
However, variations can be used, including the entire length of the called or
annotated transcripts.

<<DE for trascripts, eval=TRUE, echo=TRUE>>=
# For called transcripts
library(edgeR)
txLimit <- limitToXkb(txFinal)
ctS0mR1 <- countOverlaps(txLimit, S0mR1)
ctS0mR2 <- countOverlaps(txLimit, S0mR2)
ctS40mR1 <- countOverlaps(txLimit, S40mR1)
ctS40mR2 <- countOverlaps(txLimit, S40mR2)

pcounts <- as.matrix(data.frame(ctS0mR1, ctS0mR2, ctS40mR1, ctS40mR2))
group <- factor(c("S0m", "S0m", "S40m", "S40m"))
lib.size <- c(NROW(S0mR1), NROW(S0mR2), NROW(S40mR1), NROW(S40mR2))
d <- DGEList(counts=pcounts, lib.size=lib.size, group=group)
d <- estimateCommonDisp(d)
et <- exactTest(d)
de <- decideTests(et, p=0.001, adjust="fdr")
detags <- seq_len(NROW(d))[as.logical(de)]
# Number of transcripts regulated at 40m
cat("up: ",sum(de==1), " down: ", sum(de==-1), "\n")
@

<<fig3plot, eval=TRUE, echo=TRUE, include=FALSE>>=
plotSmear(et, de.tags=detags)
# 2 fold up or down
abline(h = c(-1,1), col="blue")
@

\begin{figure}
\begin{center}
<<label=fig3,fig=TRUE,echo=FALSE>>=
<<fig3plot>>
@
\end{center}
\caption{Transcript MAplot}
\label{Figure:three}
\end{figure}


<<DE for annotated genes, eval=TRUE, echo=TRUE, include=FALSE>>=
# For ucsc knownGenes
kgChr7 <- transcripts(kgdb, filter <- list(tx_chrom = "chr7"),
                      columns=c("gene_id", "tx_id", "tx_name"))
map <- select(org.Hs.eg.db,
        keys=unique(unlist(mcols(kgChr7)$gene_id)),
        columns=c("SYMBOL"), keytype=c("ENTREZID"))
missing <- elementNROWS(mcols(kgChr7)[,"gene_id"]) == 0
kgChr7 <- kgChr7[!missing,]

inx <- match(unlist(mcols(kgChr7)$gene_id), map$ENTREZID)
mcols(kgChr7)$symbol <- map[inx,"SYMBOL"]

kgLimit <- limitToXkb(kgChr7)
ctS0mR1 <- countOverlaps(kgLimit, S0mR1)
ctS0mR2 <- countOverlaps(kgLimit, S0mR2)
ctS40mR1 <- countOverlaps(kgLimit, S40mR1)
ctS40mR2 <- countOverlaps(kgLimit, S40mR2)

counts <- as.matrix(data.frame(ctS0mR1, ctS0mR2, ctS40mR1, ctS40mR2))
group <- factor(c("S0m", "S0m", "S40m", "S40m"))
lib.size <- c(NROW(S0mR1), NROW(S0mR2), NROW(S40mR1), NROW(S40mR2))
d <- DGEList(counts=counts, lib.size=lib.size, group=group)

d <- estimateCommonDisp(d)
et <- exactTest(d)
de <- decideTests(et, p=0.001, adjust="fdr")
detags <- seq_len(NROW(d))[as.logical(de)]
symbols <- mcols(kgChr7)$symbol
# Number of unique genes regulated at 40m
cat("up: ", NROW(unique(symbols[de==1])), "\n")
cat("down: ", NROW(unique(symbols[de==-1])), "\n")
plotSmear(et, de.tags=detags)
abline(h = c(-1,1), col="blue")
@
\subsection{Metagene Analysis}
Metagenes show the distribution of reads near TSS of a set of regulated
genes (or some other alignable genomic features of interest).
It can be thought as a smoothed average of read density weighted
by expression over the set of TSS.  The \Rfunction{runMetaGene}
function has option for sampling.  If \Rcode{TRUE}, 10\% of the
transcription units are sampled with replacement 1,000 times and median value
at each position in the transcription unit over the samples is used for final
metagene result.  Using subsampling results in an image is more robust to
outliers, especially when the size of sample is relatively small.

<<metagene analysis, eval=TRUE, keep.source=TRUE, results=hide>>=
upGenes <- kgChr7[de==1,]
expReads <- mean(c(NROW(S0m), NROW(S40m)))
# Metagene around TSS
mg0m <- runMetaGene(features=upGenes, reads=S0m, size=100,
                       normCounts=expReads/NROW(S0m), sampling=FALSE,
               mc.cores=getOption("mc.cores"))
mg40m <- runMetaGene(features=upGenes, reads=S40m, size=100,
                        normCounts=expReads/NROW(S40m), sampling=FALSE,
                mc.cores=getOption("mc.cores"))
@

<<plot metagene, eval=TRUE, keep.source=TRUE, results=hide>>=
plotMetaGene <- function(POS=c(-10000:+9999), mg, MIN, MAX){
    plot(POS, mg$sense, col="red", type="h", xlim=c(-5000, 5000),
        ylim=c(floor(MIN),ceiling(MAX)), ylab="Read Density",
        xlab="Position (relative to TSS)")
     points(POS, (-1*rev(mg$antisense)), col="blue", type="h")
     abline(mean(mg$sense[5000:8000]), 0, lty="dotted")
}

MAX <- max(c(mg0m$sense, mg40m$sense))
MIN <- -1*max(c(mg0m$antisense, mg40m$antisense))
plotMetaGene(mg=mg0m, MIN=MIN, MAX=MAX)
plotMetaGene(mg=mg40m, MIN=MIN, MAX=MAX)
@

<<metagene figures, eval=TRUE, echo=FALSE, results=hide>>=
png(filename="metagene0m.png")
plotMetaGene(mg=mg0m, MIN=MIN, MAX=MAX)
dev.off()
png(filename="metagene40m.png", width=480, height=480)
plotMetaGene(mg=mg40m, MIN=MIN, MAX=MAX)
dev.off()
@

<<save data for vignette, eval=FALSE, echo=FALSE, results=hide>>=
save(hmmResult, evals, file="Vignette.RData")
@

\begin{figure}
    \begin{center}
    \subfloat[0 min.]{\label{fig:a}
    \includegraphics[width=.45\textwidth]{metagene0m.png}}
    \qquad
    \subfloat[40 min.]{\label{fig:b}
    \includegraphics[width=.45\textwidth]{metagene40m.png}}
    \caption{Metagenes}
    \end{center}
\end{figure}


\section{Session Info}
<<sessionInfo, results=tex, print=TRUE, eval=TRUE>>=
toLatex(sessionInfo())
@

<<eval=TRUE,echo=FALSE,results=hide>>=
setwd(savewd)
@
\bibliography{groHMM}

\end{document}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fitDistributions.R
\name{RgammaMLE}
\alias{RgammaMLE}
\title{RgammaMLE fits a gamma distribution to a specified data vector using maximum
likelihood.}
\usage{
RgammaMLE(X)
}
\arguments{
\item{X}{A vector of observations, assumed to be real numbers in the
inveraval [0,+Inf).}
}
\value{
Returns a list of parameters for the best-fit gamma
distribution (shape and scale).
}
\description{
RgammaMLE fits a gamma distribution to a specified data vector using maximum
likelihood.
}
\author{
Charles G. Danko
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/detectTranscripts.R
\name{detectTranscripts}
\alias{detectTranscripts}
\title{detectTranscripts detects transcripts de novo using a two-state hidden
Markov model (HMM).}
\usage{
detectTranscripts(
  reads = NULL,
  Fp = NULL,
  Fm = NULL,
  LtProbA = -5,
  LtProbB = -200,
  UTS = 5,
  size = 50,
  threshold = 0.1,
  debug = TRUE,
  ...
)
}
\arguments{
\item{reads}{A GRanges object representing a set of mapped reads.}

\item{Fp}{Wiggle-formatted read counts on "+" strand. Optionally, Fp and Fm
represent list() filled with a vector of counts for each chromosome.
Can detect transcripts starting from a fixed-step wiggle.}

\item{Fm}{Wiggle-formatted read counts on "-" strand.}

\item{LtProbA}{Log probability of t... .  Default: -5. One of these is just
an initialization, and the final value is set by EM.  The other is a holdout
parameter.}

\item{LtProbB}{Log probability of t... .  Default: -200.}

\item{UTS}{Varience in read counts of the untranscribed sequence.
Default: 5.}

\item{size}{Log probability of t... .  Default: -5.}

\item{threshold}{Threshold change in total likelihood, below which EM exits.}

\item{debug}{If set to TRUE, provides additional print options.
Default: FALSE}

\item{...}{Extra argument passed to mclapply}
}
\value{
Returns a list of emisParams, trnasParams, viterbiStates, and
transcripts.  The transcript element is a GRanges object representing the
predicted genomic coordinates of transcripts on both the + and - strand.
}
\description{
Read counts can be specified as either a GRanges object (reads), or using a
fixed-step wiggle-format passed in a list (Fp and Fm).
Either reads or BOTH Fp and Fm must be specified.
}
\details{
Supports parallel processing using mclapply in the 'parallel' package.
To change the number of processors set the option 'mc.cores'.

Reference: Hah N, Danko CG, Core L, Waterfall JJ, Siepel A, Lis JT,
Kraus WL. A rapid, extensive, and transient transcriptional response to
estrogen signaling in breast cancer cells. Cell. 2011 May 13;145(4):622-34.
doi: 10.1016/j.cell.2011.03.042.
}
\examples{
S0mR1 <- as(readGAlignments(system.file("extdata", "S0mR1.bam",
               package="groHMM")), "GRanges")
## Not run:
# hmmResult <- detectTranscripts(S0mR1, LtProbB=-200, UTS=5, threshold=1)
# txHMM <- hmmResult$transcripts
}
\author{
Charles G. Danko and Minho Chae
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/windowAnalysis.R
\name{windowAnalysis}
\alias{windowAnalysis}
\title{windowAnalysis Returns a vector of integers representing the counts of
reads in a moving window.}
\usage{
windowAnalysis(
  reads,
  strand = "*",
  windowSize = stepSize,
  stepSize = windowSize,
  chrom = NULL,
  limitPCRDups = FALSE,
  ...
)
}
\arguments{
\item{reads}{GenomicRanges object representing the position of reads
mapping in the genome.}

\item{strand}{Takes values of "+", "-", or "*".  "*" denotes collapsing
reads on both strands.  Default: "*".}

\item{windowSize}{Size of the moving window. Either windowSize or
stepSize must be specified.}

\item{stepSize}{The number of bp moved with each step.}

\item{chrom}{Chromosome for which to return data.
Default: returns all avaliable data.}

\item{limitPCRDups}{Counts only one read mapping to each start site.
NOTE: If set to TRUE, assumes that all reads are the same length
(don't use for paired-end data).  Default: FALSE.}

\item{...}{Extra argument passed to mclapply}
}
\value{
Returns a list object, each element of which represents a
chromosome.
}
\description{
Supports parallel processing using mclapply in the 'parallel' package.
To change the number of processors, set the option 'mc.cores'.
}
\examples{
S0mR1 <- as(readGAlignments(system.file("extdata", "S0mR1.bam",
     package="groHMM")), "GRanges")
## Not run:
# Fp <- windowAnalysis(S0mR1, strand="+", windowSize=50)
}
\author{
Charles G. Danko and Minho Chae
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metaGene.R
\name{metaGene_nL}
\alias{metaGene_nL}
\title{Returns a histogram of the number of reads in each section of a moving
window of #' variable size across genes.}
\usage{
metaGene_nL(features, reads, n_windows = 1000, debug = FALSE, ...)
}
\arguments{
\item{features}{A GRanges object representing a set of genomic coordinates.}

\item{reads}{A GRanges object representing a set of mapped reads.}

\item{n_windows}{The number of windows to break genes into.}

\item{debug}{If set to TRUE, provides additional print options.
Default: FALSE}

\item{...}{Extra argument passed to mclapply}
}
\value{
Returns a vector representing the 'typical' signal across genes of
different length.
}
\description{
Supports parallel processing using mclapply in the 'parallel' package.
To change the number of processors, use the argument 'mc.cores'.
}
\examples{
features <- GRanges("chr7", IRanges(1000, 1100), strand = "+")
reads <- GRanges("chr7", IRanges(start = c(1000:1004, 1100),
                                 width = rep(1, 6)), strand="+")
metaGene_nL(features, reads, n_windows=5)
}
\author{
Charles G. Danko and Minho Chae
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fitDistributions.R
\name{Rnorm.exp}
\alias{Rnorm.exp}
\title{Rnorm.exp fits a normal+exponential distribution to a specified data
vector using maximum likelihood.}
\usage{
Rnorm.exp(
  xi,
  wi = rep(1, NROW(xi)),
  guess = c(0.5, 0, 1, 1),
  tol = sqrt(.Machine$double.eps),
  maxit = 10000
)
}
\arguments{
\item{xi}{A vector of observations, assumed to be real numbers in the
inveraval (-Inf,+Inf).}

\item{wi}{A vector of weights.  Default: vector of repeating 1; indicating
all observations are weighted equally. (Are these normalized internally?!
Or do they have to be [0,1]?)}

\item{guess}{Initial guess for paremeters.  Default: c(0.5, 0, 1, 1).}

\item{tol}{Convergence tolerance.  Default: sqrt(.Machine$double.eps).}

\item{maxit}{Maximum number of iterations.  Default: 10,000.}
}
\value{
Returns a list of parameters for the best-fit normal distribution
(alpha, mean, varience, and lambda).
}
\description{
Distrubtion function devined by: alpha*Normal(mean, varience)+(1-alpha)
*Exponential(lambda).
}
\details{
Fits nicely with data types that look normal overall, but have a long
tail starting for positive values.
}
\author{
Charles G. Danko
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/limitToXkb.R
\name{readBed}
\alias{readBed}
\title{readBed Returns a GenomicRanges object constrcuted from the specified bed
file.}
\usage{
readBed(file, ...)
}
\arguments{
\item{file}{Path to the input file.}

\item{...}{Extra argument passed to read.table}
}
\value{
Returns GRanges object representing mapped reads.
}
\description{
Bed file format is assumed to be either four column: seqnames, start, end,
strand columns; or six column: seqnames, start, end, name, score, and strand.
Three column format is also possible when there is no strand information.
}
\details{
Any additional arguments availiable to read.table can be specified.
}
\examples{
fileBed <- system.file(package = "rtracklayer", "tests", "test.bed")
readBed(fileBed, skip = 2, colClasses = c(rep(NA, 6), rep("NULL", 6)))
}
\author{
Minho Chae and Charles G. Danko.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/limitToXkb.R
\name{countMappableReadsInInterval}
\alias{countMappableReadsInInterval}
\title{countMappableReadsInInterval counts the number of mappable reads in a set
of genomic features.}
\usage{
countMappableReadsInInterval(features, UnMap, debug = FALSE, ...)
}
\arguments{
\item{features}{A GRanges object representing a set of genomic coordinates.
The meta-plot will be centered on the start position.}

\item{UnMap}{List object representing the position of un-mappable reads.
Default: not used.}

\item{debug}{If set to TRUE, provides additional print options.
Default: FALSE}

\item{...}{Extra argument passed to mclapply}
}
\value{
Returns a vector of counts, each representing the number of reads
inside each genomic interval.
}
\description{
Supports parallel processing using mclapply in the 'parallel' package.
To change the number of processors, use the argument 'mc.cores'.
}
\author{
Charles G. Danko and Minho Chae
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getTxDensity.R
\name{getTxDensity}
\alias{getTxDensity}
\title{getTxDensity Calculates transcript density.}
\usage{
getTxDensity(
  tx,
  annox,
  plot = TRUE,
  scale = 1000L,
  nSampling = 0L,
  samplingRatio = 0.1,
  ...
)
}
\arguments{
\item{tx}{GRanges of transcripts.}

\item{annox}{GRanges of non-overlapping annotatoins.}

\item{plot}{Logical.  If TRUE, plot transcript density.  Default: TRUE}

\item{scale}{Numeric. Scaled size of a gene for transcript density
calculation.
Default: 1000L}

\item{nSampling}{Numeric. Number of subsampling.  Default: 0L}

\item{samplingRatio}{Numeric. Ratio of sampling for annotations.
Default: 0.1}

\item{...}{Extra argument passed to mclapply.}
}
\value{
Returns a list of FTD, TTD, PostTTS, and TUA.
}
\description{
Calculates transcript density for transcripts which overlapps with
annotations.
For 'run genes together' or 'broken up a single annotation' errors,
best overlapped transcripts or annotations are used.
}
\details{
Supports parallel processing using mclapply in the 'parallel' package.
To change the number of processors
set the option 'mc.cores'.
}
\examples{
tx <- GRanges("chr7", IRanges(start=seq(1000,4000, by=1000),
width=seq(1000, 1300, by=100)), strand=rep("+", 4))
annox <- GRanges("chr7", IRanges(start=seq(1100,4100, by=1000),
width=seq(900, 1200, by=100)), strand=rep("+", 4))
## Not run:
# density <- getTxDensity(tx, annox)
}
\author{
Minho Chae
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/breakTranscriptsOnGenes.R
\name{combineTranscripts}
\alias{combineTranscripts}
\title{combineTranscripts Combines transnscipts.}
\usage{
combineTranscripts(tx, annox, geneSize = 1000, threshold = 0.8, plot = FALSE)
}
\arguments{
\item{tx}{GRanges of transcripts.}

\item{annox}{GRanges of non-overlapping annotations for reference.}

\item{geneSize}{Numeric. Minimum gene size in annotations to be used as
reference.
Default: 1000}

\item{threshold}{Numeric. Ratio of overlapped region relative to transcript
width.
Transcripts only greater than this threshold are subjected to be combined.
Default: 0.8}

\item{plot}{Logical.  If set to TRUE, show easch step in a plot.
Default: FALSE}
}
\value{
Returns GRanges object of combined transcripts.
}
\description{
Combines transcripts  that are within the same gene annotation, combining
smaller transcripts for genes
 with low regulation into a single transcript representing the gene.
}
\examples{
tx <- GRanges("chr7", IRanges(start=c(1000, 20000), width=c(10000,10000)),
strand="+")
annox <- GRanges("chr7", IRanges(1000, 30000), strand="+")
combined <- combineTranscripts(tx, annox)
}
\author{
Minho Chae and Charles G. Danko
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/polymeraseWave.R
\name{polymeraseWave}
\alias{polymeraseWave}
\title{Given GRO-seq data, identifies the location of the polymerase 'wave' in
up- or down- regulated genes.}
\usage{
polymeraseWave(
  reads1,
  reads2,
  genes,
  approxDist,
  size = 50,
  upstreamDist = 10000,
  TSmooth = NA,
  NonMap = NULL,
  prefix = NULL,
  emissionDistAssumption = "gamma",
  finterWindowSize = 10000,
  limitPCRDups = FALSE,
  returnVal = "simple",
  debug = TRUE
)
}
\arguments{
\item{reads1}{Mapped reads in time point 1.}

\item{reads2}{Mapped reads in time point 2.}

\item{genes}{A set of genes in which to search for the wave.}

\item{approxDist}{The approximate position of the wave.  Suggest using 2000
[bp/ min] * time [min], for mammalian data.}

\item{size}{The size of the moving window. Suggest using 50 for direct
ligation data, and 200 for circular ligation data.  Default: 50.}

\item{upstreamDist}{The amount of upstream sequence to include Default: 10
kb.}

\item{TSmooth}{Optimonally, outlying windows are set a maximum value over
the inter-quantile interval, specified by TSmooth.  Reasonable value: 20;
Default: NA (for no smoothing).  Users are encouraged to use this
parameter ONLY in combination with the normal distribution assumptions.}

\item{NonMap}{Optionally, un-mappable positions are trated as missing data.
NonMap passes in the list() structure for un-mappable regions.}

\item{prefix}{Optionally, writes out png images of each gene examined for a
wave.  'Prefix' denotes the file prefix for image names written to disk.
Users are encouraged to create a new directory and write in a full path.}

\item{emissionDistAssumption}{Takes values "norm", "normExp", and "gamma".
Specifies the functional form of the 'emission' distribution for states I
and II (i.e. 5' of the gene, and inside of the wave).  In our experience,
"gamma" works best for highly-variable 'spikey' data, and "norm" works for
smooth data.  As a general rule of thumb, "gamma" is used for libraries
made using the direct ligation method, and "norm" for circular ligation
data.  Default: "gamma".}

\item{finterWindowSize}{Method returns 'quality' information for each gene
to which a wave was fit.  Included in these metrics are several that
define a moving window.  The moving window size is specified by
filterWindowSize.  Default: 10 kb.}

\item{limitPCRDups}{If true, counts only 1 read at each position with >= 1
read.  NOT recommended to set this to TRUE.  Defulat: FALSE.}

\item{returnVal}{Takes value "simple" (default) or "alldata". "simple"
returns a data.frame with Pol II wave end positions.  "alldata" returns
all of the availiable data from each gene, including the full posterior
distribution of the model after EM.}

\item{debug}{If TRUE, prints error messages.}
}
\value{
Returns either a data.frame with Pol II wave end positions, or a
  List() structure with additional data, as specified by returnVal.
}
\description{
The model is a three state hidden Markov model (HMM).  States represent:
(1) the 5' end of genes upstream of the transcription start site,
(2) upregulated sequence, and (3) the 3' end of the gene through the
polyadenylation site.
}
\details{
The model computes differences in read counts between the two conditions.
Differences are assumed fit a functional form which can be specified by
the user (using the emissionDistAssumption argument).
Currently supported functional forms include a normal distribution
(good for GRO-seq data prepared using the circular ligation protocol),
a gamma distribution (good for 'spikey' ligation based GRO-seq data),
and a long-tailed normal+exponential distribution was implemented, but
never deployed.

Initial parameter estimates are based on initial assumptions of
transcription rates taken from the literature.  Subsequently all parameters
are fit using Baum-Welch expetation maximization.

Reference: Danko CG, Hah N, Luo X, Martins AL, Core L, Lis JT, Siepel A,
Kraus WL. Signaling Pathways Differentially Affect RNA Polymerase II
Initiation, Pausing, and Elongation Rate in Cells. Mol Cell.
2013 Mar 19. doi:pii: S1097-2765(13)00171-8. 10.1016/j.molcel.2013.02.015.

Arguments:
}
\examples{
genes <- GRanges("chr7", IRanges(2394474,2420377), strand="+",
SYMBOL="CYP2W1", ID="54905")
reads1 <- as(readGAlignments(system.file("extdata", "S0mR1.bam",
                            package="groHMM")), "GRanges")
reads2 <- as(readGAlignments(system.file("extdata", "S40mR1.bam",
                            package="groHMM")), "GRanges")
approxDist <- 2000*10
# Not run:
# pw <- polymeraseWave(reads1, reads2, genes, approxDist)
}
\author{
Charles G. Danko
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getTxDensity.R
\name{evaluateHMMInAnnotations}
\alias{evaluateHMMInAnnotations}
\title{evaluateHMM Evaluates HMM calling.}
\usage{
evaluateHMMInAnnotations(tx, annox)
}
\arguments{
\item{tx}{GRanges of transcripts predicted by HMM.}

\item{annox}{GRanges of non-overlapping annotatoins.}
}
\value{
a list of error information; merged annotations, dissociated annotation,
total, and rate.
}
\description{
Evaluates HMM calling of transripts compared to known annotations.
}
\examples{
tx <- GRanges("chr7", IRanges(start=seq(100, 1000, by=200),
width=seq(100, 1000, by=100)), strand="+")
annox <- GRanges("chr7", IRanges(start=seq(110, 1100, by=150),
width=seq(100, 1000, by=150)), strand="+")
error <- evaluateHMMInAnnotations(tx, annox)
}
\author{
Minho Chae
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/writeWiggle.R
\name{writeWiggle}
\alias{writeWiggle}
\title{writeWiggle writes a wiggle track or BigWig file suitable for uploading
to the UCSC genome browser.}
\usage{
writeWiggle(
  reads,
  file,
  strand = "*",
  fileType = "wig",
  size = 50,
  normCounts = NULL,
  reverse = FALSE,
  seqinfo = NULL,
  track.type.line = FALSE,
  ...
)
}
\arguments{
\item{reads}{GenomicRanges object representing the position of reads
mapping in the genome.}

\item{file}{Specifies the filename for output.}

\item{strand}{Takes values of "+", "-", or "*".  Computes Writes a wiggle
on the speicified strand.  "*" denotes collapsing reads on both strands.
Default: "*".}

\item{fileType}{Takes values of "wig" or "BigWig". Default: "wig".}

\item{size}{Size of the moving window.}

\item{normCounts}{A normalization factor correcting for library size
or other effects.  For example, total mappible read counts might be a
reasonable value.  Default: 1 (i.e. no normalization).}

\item{reverse}{If set to TRUE, multiplies values by -1.
Used for reversing GRO-seq data on the negative (-) strand. Default: FALSE}

\item{seqinfo}{Seqinfo object for reads. Default: NULL.}

\item{track.type.line}{If set to TRUE, prints a header identifying the
file as a wiggle.  Necessary to upload a custom track to the UCSC
genome browser.  Default: TRUE}

\item{...}{Extra argument passed to mclapply.}
}
\description{
writeWiggle writes a wiggle track or BigWig file suitable for uploading
to the UCSC genome browser.
}
\examples{
S0mR1 <- as(readGAlignments(system.file("extdata", "S0mR1.bam",
package="groHMM")), "GRanges")
## Not run:
# writeWiggle(reads=S0mR1, file="S0mR1_Plus.wig", fileType="wig",
# strand="+", reverse=FALSE)
}
\author{
Minho Chae and Charles G. Danko
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tlsFits.R
\name{tlsDeming}
\alias{tlsDeming}
\title{A 'total least squares' implementation using demming regression.}
\usage{
tlsDeming(x, y, d = 1)
}
\arguments{
\item{x}{X values.}

\item{y}{Y values.}

\item{d}{Ratio of variences. Default: 1, for orthogonal regression.}
}
\value{
Parameters for the linear model.
}
\description{
A 'total least squares' implementation using demming regression.
}
\author{
Charles G. Danko
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tlsFits.R
\name{tlsLoess}
\alias{tlsLoess}
\title{A 'total least squares'-like hack for LOESS.  Works by rotating points
45 degrees, fitting LOESS, and rotating back.}
\usage{
tlsLoess(x, y, theta = -pi/4, span = 1)
}
\arguments{
\item{x}{X values.}

\item{y}{Y values.}

\item{theta}{Amount to rotate, sets the ratio of variences that are assumed
by the hack.  Default: -pi/4 radians (45 degrees) for orthogonal regression.}

\item{span}{The LOESS span parameter.  Default: 1}
}
\value{
List of input values and LOESS predictions.
}
\description{
A 'total least squares'-like hack for LOESS.  Works by rotating points
45 degrees, fitting LOESS, and rotating back.
}
\author{
Charles G. Danko
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metaGene.R
\name{runMetaGene}
\alias{runMetaGene}
\title{Runs metagene analysis for sense and antisense direction.}
\usage{
runMetaGene(
  features,
  reads,
  anchorType = "TSS",
  size = 100L,
  normCounts = 1L,
  up = 10000L,
  down = NULL,
  sampling = FALSE,
  nSampling = 1000L,
  samplingRatio = 0.1,
  ...
)
}
\arguments{
\item{features}{GRanges A GRanges object representing a set of genomic
coordinates, i.e., set of genes.}

\item{reads}{GRanges of reads.}

\item{anchorType}{Either 'TSS' or 'TTS'.  Metagene will be centered on the
transcription start site(TSS) or transcription termination site(TTS).
Default: TSS.}

\item{size}{Numeric.  The size of the moving window. Default: 100L}

\item{normCounts}{Numeric.  Normalization vector such as average reads.
Default: 1L}

\item{up}{Numeric. Distance upstream of each feature to align and histogram.
Default: 1 kb}

\item{down}{Numeric. Distance downstream of each feature to align and
histogram.  If NULL, down is same as up. Default: NULL}

\item{sampling}{Logical.  If TRUE, subsampling of Metagene is used.
Default: FALSE}

\item{nSampling}{Numeric. Number of subsampling.  Default: 1000L}

\item{samplingRatio}{Numeric. Ratio of sampling for features.  Default: 0.1}

\item{...}{Extra argument passed to mclapply.}
}
\value{
A list of integer-Rle for sense and antisene.
}
\description{
Supports parallel processing using mclapply in the 'parallel' package.
To change the number of processors, set the option 'mc.cores'.
}
\examples{
features <- GRanges("chr7", IRanges(start=1000:1001, width=rep(1,2)),
 strand=c("+", "-"))
reads <- GRanges("chr7", IRanges(start=c(1000:1003, 1100:1101),
 width=rep(1, 6)), strand=rep(c("+","-"), 3))
## Not run:
# mg <- runMetaGene(features, reads, size=4, up=10)
}
\author{
Minho Chae
}

\name{groHMM-package}
\alias{groHMM-package}
\alias{groHMM}
\docType{package}
\title{
groHMM: GRO-seq Analysis Pipeline
}
\description{
groHMM was developed for analysis of GRO-seq data, which provides a genome wide 'map' of the position and orientation of all
transcriptionally active RNA polymerases. groHMM predicts the boundaries of
transcriptional activity across the genome de novo using a two-state hidden Markov
model (HMM).  The model essentially divides the genome into 'transcribed' and
'non-transcribed' regions in a strand specific manner. 

We also use HMMs to identify the leading edge of Pol II at genes activated by a
stimulus in GRO-seq time course data.  This approach allows the genome-wide
interrogation of transcription rates in cells.

In addition to these advanced features, groHMM provides wrapper functions for
counting raw reads, generating wiggle files for visualization,
and creating metagene (averaging) plots.  Although groHMM is tailored
towards GRO-seq data, the same functions and analytical methodologies can, in principal,
be applied to a wide variety of other short read data sets.
}
\details{
\tabular{ll}{
Package: \tab groHMM\cr
Type: \tab Package\cr
Version: \tab 0.99.0\cr
Date: \tab 2014-04-02\cr
License: \tab GPL (>=3)\cr
LazyLoad: \tab yes\cr
Depends: \tab R (>= 2.14.0), MASS, GenomicRanges, rtracklayer, parallel\cr
}
}
\author{
Charles G. Danko, Minho Chae, Andre Martins  \cr

Maintainer: Minho Chae<minho.chae@gmail.com>
}
\references{
Luo, X., Chae, M., Krishnakumar, R,, Danko, C., Kraus, L. Dynamic reorganization of the AC16 cardiomyocyte transcriptome in response to TNFa
        signaling revealed by integrated genomic analyses. BMC Genomics. 2014 Feb 24;15(1):155

Hah, N., Danko, C., Core, L., Waterfall, J., Siepel, A., Lis, J., Kraus, L.  A Rapid, Extensive, and Transient Transcriptional 
Response to Estrogen Signaling in Breast Cancer Cells. Cell. 2011 May 13;145(4):622-34 
}
\keyword{ package }

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tlsFits.R
\name{tlsSvd}
\alias{tlsSvd}
\title{A 'total least squares' implementation using singular value demposition.}
\usage{
tlsSvd(x, y)
}
\arguments{
\item{x}{X values.}

\item{y}{Y values.}
}
\value{
Parameters for the linear model Y~a*X+e.
}
\description{
A 'total least squares' implementation using singular value demposition.
}
\author{
Charles G. Danko
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeConsensusAnnotations.R
\name{makeConsensusAnnotations}
\alias{makeConsensusAnnotations}
\title{makeConsensusAnnotations Makes a consensus annotation}
\usage{
makeConsensusAnnotations(ar, minGap = 1L, minWidth = 1000L, ...)
}
\arguments{
\item{ar}{GRanges of annotations to be collapsed.}

\item{minGap}{Minimun gap between overlapped annotations after truncated.
Default: 1L}

\item{minWidth}{Minimun width of consensus annotations. Default: 1000L}

\item{...}{Extra argument passed to mclapply.}
}
\value{
Returns GRanges object of annotations.
}
\description{
Makes a non-overlapping consensus annotation.  Gene annotations are often
overalpping due to #' multiple isoforms for a gene.
In consensus annotation, isoforms are first reduced so that only
redundant intervals are used to represent a genomic interval for a gene,
i.e., a gene id.
Remaining unresolved annotations are further reduced by truncating 3'
end of annotations.
}
\details{
Supports parallel processing using mclapply in the 'parallel' package.
To change the number of processors, use the argument 'mc.cores'.
}
\examples{
library(TxDb.Hsapiens.UCSC.hg19.knownGene)
txdb <- TxDb.Hsapiens.UCSC.hg19.knownGene
tx <- transcripts(txdb, columns = c("gene_id", "tx_id", "tx_name"),
                  filter = list(tx_chrom = "chr7"))
tx <- tx[grep("random", as.character(seqnames(tx)), invert=TRUE), ]
## Not run:
# ca <- makeConsensusAnnotations(tx)
}
\author{
Minho Chae
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metaGene.R
\name{metaGene}
\alias{metaGene}
\title{Returns a histogram of the number of reads in each section of a moving
window centered on a certain feature.}
\usage{
metaGene(
  features,
  reads = NULL,
  plusCVG = NULL,
  minusCVG = NULL,
  size = 100L,
  up = 10000L,
  down = NULL,
  ...
)
}
\arguments{
\item{features}{A GRanges object representing a set of genomic coordinates.
The meta-plot will be centered on the transcription start site (TSS)}

\item{reads}{A GRanges object representing a set of mapped reads.
Instead of 'reads', 'plusCVG' and 'minusCVG' can be used  Default: NULL}

\item{plusCVG}{An IntegerRangesList object for reads with '+' strand.}

\item{minusCVG}{An IntegerRangesList object for reads with '-' strand.}

\item{size}{The size of the moving window.}

\item{up}{Distance upstream of each features to align and histogram.
Default: 10 kb.}

\item{down}{Distance downstream of each features to align and histogram.
If NULL, same as up. Default: NULL.}

\item{...}{Extra argument passed to mclapply}
}
\value{
Returns a integer-Rle representing the 'typical' signal
centered on a point of interest.
}
\description{
Supports parallel processing using mclapply in the 'parallel' package.
To change the number of processors, set the option 'mc.cores'.
}
\examples{
features <- GRanges("chr7", IRanges(1000, 1000), strand="+")
reads <- GRanges("chr7", IRanges(start=c(1000:1004, 1100),
 width=rep(1, 6)), strand="+")
mg <- metaGene(features, reads, size=4, up=10)
}
\author{
Charles G. Danko and Minho Chae
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fitDistributions.R
\name{Rnorm}
\alias{Rnorm}
\title{Rnorm fits a normal distribution to a specified data vector using maximum
likelihood.}
\usage{
Rnorm(X)
}
\arguments{
\item{X}{A vector of observations, assumed to be real numbers in the
inveraval (-Inf,+Inf).}
}
\value{
Returns a list of parameters for the best-fit normal distribution
(mean and varience).
}
\description{
Rnorm fits a normal distribution to a specified data vector using maximum
likelihood.
}
\author{
Charles G. Danko
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\name{getCores}
\alias{getCores}
\title{Returns the number of cores.}
\usage{
getCores(cores)
}
\arguments{
\item{cores}{the number of cores, it is 1 in windows platform.}
}
\description{
Returns the number of cores.
}
\examples{
cores <- getCores(2L)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/expressedGenes.R
\name{expressedGenes}
\alias{expressedGenes}
\title{Function identifies expressed features using the methods introduced in
Core, Waterfall, Lis; Science, Dec. 2008.}
\usage{
expressedGenes(
  features,
  reads,
  Lambda = NULL,
  UnMap = NULL,
  debug = FALSE,
  ...
)
}
\arguments{
\item{features}{A GRanges object representing a set of genomic coordinates.
The meta-plot will be centered on the start position.
There can be optional "ID" column for gene ids.}

\item{reads}{A GRanges object representing a set of mapped reads.}

\item{Lambda}{Measurement of assay noise.  Default: 0.04 reads/ kb in a
library of 10,751,533 mapped reads. (background computed in Core,
Waterfall, Lis. (2008) Science.).}

\item{UnMap}{List object representing the position of un-mappable reads.
Default: not used.}

\item{debug}{If set to true, returns the number of positions.
Default: FALSE.}

\item{...}{Extra argument passed to mclapply}
}
\value{
Returns a data.frame representing the expression p.values for
features of interest.
}
\description{
Supports parallel processing using mclapply in the 'parallel' package.
To change the number of processors use the argument 'mc.cores'.
}
\examples{
features <- GRanges("chr7", IRanges(2394474, 2420377), strand = "+",
                    ID = "gene1")
reads <-
    system.file("extdata", "S0mR1.bam", package = "groHMM") |>
    readGAlignments() |>
    as("GRanges")
expressedGenes(features, reads)
}
\author{
Charles G. Danko
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pausingIndex.R
\name{pausingIndex}
\alias{pausingIndex}
\title{Returns the pausing index for different genes.  TODO: DESCRIBE THE PAUSING
INDEX.}
\usage{
pausingIndex(
  features,
  reads,
  size = 50,
  up = 1000,
  down = 1000,
  UnMAQ = NULL,
  debug = FALSE,
  ...
)
}
\arguments{
\item{features}{A GRanges object representing a set of genomic coordinates.}

\item{reads}{A GRanges object representing a set of mapped reads.}

\item{size}{The size of the moving window.}

\item{up}{Distance upstream of each f to align and histogram.}

\item{down}{Distance downstream of each f to align and histogram (NULL).}

\item{UnMAQ}{Data structure representing the coordinates of all un-mappable
regions in the genome.}

\item{debug}{If set to TRUE, provides additional print options.  Default:
FALSE}

\item{...}{Extra argument passed to mclapply}
}
\value{
Returns a data.frame of the pausing indices for the input genes.

Returns the pausing index for different genes.
}
\description{
Supports parallel processing using mclapply in the 'parallel' package.
To change the number of processors, use the argument 'mc.cores'.
}
\examples{
features <- GRanges("chr7", IRanges(2394474,2420377), strand="+")
reads <- as(readGAlignments(system.file("extdata", "S0mR1.bam",
            package="groHMM")), "GRanges")
## Not run:
# pi <- pausingIndex(features, reads)
}
\author{
Charles G. Danko and Minho Chae.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metaGene.R
\name{averagePlot}
\alias{averagePlot}
\title{Returns the average profile of tiling array probe intensity values or
wiggle-like count data centered on a set of genomic positions
(specified by 'Peaks').}
\usage{
averagePlot(ProbeData, Peaks, size = 50, bins = seq(-1000, 1000, size))
}
\arguments{
\item{ProbeData}{Data.frame representing chromosome, window center,
and a value.}

\item{Peaks}{Data.frame representing chromosome, and window center.}

\item{size}{Numeric.  The size of the moving window. Default: 50 bp.}

\item{bins}{The bins of the meta gene -- i.e. the number of moving windows
to break it into. Default +/- 1kb from center.}
}
\value{
A vector representing the 'typical' signal centered on the peaks of
interest.
}
\description{
Supports parallel processing using mclapply in the 'parallel' package.
To change the number of processors, use the argument 'mc.cores'.
}
\author{
Charles G. Danko and Minho Chae
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/breakTranscriptsOnGenes.R
\name{breakTranscriptsOnGenes}
\alias{breakTranscriptsOnGenes}
\title{breakTranscriptsOnGenes Breaks transcripts on genes}
\usage{
breakTranscriptsOnGenes(
  tx,
  annox,
  strand = "+",
  geneSize = 5000,
  threshold = 0.8,
  gap = 5,
  plot = FALSE
)
}
\arguments{
\item{tx}{GRanges of transcripts.}

\item{annox}{GRanges of non-overlapping annotations for reference.}

\item{strand}{Takes "+" or "-" Default: "+"}

\item{geneSize}{Numeric. Minimum gene size in annox to be used as reference.
Default: 5000}

\item{threshold}{Numeric. Ratio of overlapped region relative to a gene
width.
Transcripts only greater than this threshold are subjected to be broken.
Default: 0.8}

\item{gap}{Numeric.  Gap (bp) between broken transcripts.  Default: 5}

\item{plot}{Logical.  If set to TRUE, show each step in a plot.
Default: FALSE}
}
\value{
Returns GRanges object of broken transcripts.
}
\description{
Breaks transcripts when they are overlapped with multiple well annotated
genes.
}
\examples{
tx <- GRanges("chr7", IRanges(1000, 30000), strand="+")
annox <- GRanges("chr7", IRanges(start=c(1000, 20000),
             width=c(10000,10000)), strand="+")
bPlus <- breakTranscriptsOnGenes(tx, annox, strand="+")
}
\author{
Minho Chae and Charles G. Danko
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/limitToXkb.R
\name{limitToXkb}
\alias{limitToXkb}
\title{limitToXkb truncates a set of genomic itnervals at a constant, maximum size.}
\usage{
limitToXkb(features, offset = 1000, size = 13000)
}
\arguments{
\item{features}{A GRanges object representing a set of genomic coordinates.
The meta-plot will be centered on the start position.}

\item{offset}{Starts the interval from this position relative to the start
of each genomic features.}

\item{size}{Specifies the size of the window.}
}
\value{
Returns GRanges object with new genomic coordiates.
}
\description{
limitToXkb truncates a set of genomic itnervals at a constant, maximum size.
}
\examples{
tx <- GRanges("chr7", IRanges(1000, 30000), strand="+")
newTX <- limitToXkb(tx)
}
\author{
Minho Chae and Charles G. Danko
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metaGene.R
\name{metaGeneMatrix}
\alias{metaGeneMatrix}
\title{Returns a matrix, with rows representing read counts across a specified
gene, or other features of interest.}
\usage{
metaGeneMatrix(
  features,
  reads,
  size = 50,
  up = 1000,
  down = up,
  debug = FALSE,
  ...
)
}
\arguments{
\item{features}{A GRanges object representing a set of genomic coordinates.}

\item{reads}{A GRanges object representing a set of mapped reads.}

\item{size}{The size of the moving window.}

\item{up}{Distance upstream of each f to align and histogram Default: 1 kb.}

\item{down}{Distance downstream of each f to align and histogram
Default: same as up.}

\item{debug}{If set to TRUE, provides additional print options.
Default: FALSE}

\item{...}{Extra argument passed to mclapply}
}
\value{
Returns a vector representing the 'typical' signal across
genes of different length.
}
\description{
Supports parallel processing using mclapply in the 'parallel' package.
To change the number of processors, use the argument 'mc.cores'.
}
\author{
Charles G. Danko and Minho Chae
}
